import sys
import copy
import operator
from functools import reduce

from collections import defaultdict

import numpy

from AnyQt.QtWidgets import (
    QWidget, QVBoxLayout, QLabel, QHeaderView, QListView, QScrollArea,
    QSizePolicy, QSpacerItem, QWidgetItem
)
from AnyQt.QtGui import QColor, QPalette, QStandardItemModel, QStandardItem
from AnyQt.QtCore import Qt, QSize, QItemSelectionModel, QItemSelection

import Orange.data
import Orange.misc
from Orange.preprocess import transformation

from Orange.widgets import widget, gui, settings
from Orange.widgets.utils import itemmodels

from ..utils.group import \
    separate_by, data_type, linearize, dist_pcorr, dist_eucl, dist_spearman

from .utils.settings import SetContextHandler


def clone_attr(attr):
    newvar = attr.copy(compute_value=transformation.Identity(attr))
    newvar.attributes = copy.copy(attr.attributes)
    return newvar


def name_gen(a):
    i = 1
    while True:
        yield a + " {0}".format(i)
        i += 1

missing_name_gen = name_gen("!!missing")
inactive_name_gen = name_gen("!!inactive")


class MyHeaderView(QHeaderView):
    def mouseMoveEvent(self, event):
        event.ignore()

    def wheelEvent(self, event):
        event.ignore()


class OWGenotypeDistances(widget.OWWidget):
    name = "Expression Profile Distances"
    description = ("Compute distances between expression profiles of "
                   "different experimental factors.")
    icon = "../widgets/icons/GenotypeDistances.svg"
    priority = 1050

    inputs = [("Data", Orange.data.Table, "set_data")]
    outputs = [("Distances", Orange.misc.DistMatrix),
               ("Sorted Data", Orange.data.Table)]

    settingsHandler = SetContextHandler()

    separate_keys = settings.ContextSetting({})
    relevant_keys = settings.ContextSetting({})

    distance_measure = settings.Setting(0)
    auto_commit = settings.Setting(False)

    DISTANCE_FUNCTIONS = [
        ("Distance from Pearson correlation", dist_pcorr),
        ("Euclidean distance", dist_eucl),
        ("Distance from Spearman correlation", dist_spearman)
    ]

    def __init__(self, parent=None):
        super().__init__(self, parent)

        self.data = None
        self.partitions = []
        self.matrix = None
        self.split_groups = []
        self._disable_updates = False

        ########
        # GUI
        ########

        box = gui.widgetBox(self.controlArea, "Input")

        self.info_box = gui.widgetLabel(box, "No data on input\n")

        box = gui.widgetBox(self.controlArea, "Separate By",
                            addSpace=True)

        self.separate_view = QListView(
            selectionMode=QListView.MultiSelection
        )
        box.layout().addWidget(self.separate_view)

        box = gui.widgetBox(self.controlArea, "Sort By",
                            addSpace=True)
        self.relevant_view = QListView(
            selectionMode=QListView.MultiSelection)

        box.layout().addWidget(self.relevant_view)

        self.distance_view = gui.comboBox(
            self.controlArea, self, "distance_measure",
            box="Distance Measure",
            items=[name for name, _ in self.DISTANCE_FUNCTIONS])

        gui.rubber(self.controlArea)

        gui.auto_commit(self.controlArea, self, "auto_commit", "Commit")
        self.groups_box = gui.widgetBox(self.mainArea, "Groups")
        self.groups_scroll_area = QScrollArea()
        self.groups_box.layout().addWidget(self.groups_scroll_area)

    def sizeHint(self):
        return QSize(800, 600)

    def clear(self):
        self.data = None
        self.partitions = []
        self.split_groups = []
        self.matrix = None

    def get_suitable_keys(self, data):
        """Return suitable attr label keys from the data where the key has at least
        two unique values in the data.

        """
        attrs = [attr.attributes.items() for attr in data.domain.attributes]
        attrs = reduce(operator.iadd, attrs, [])
        # in case someone put non string values in attributes dict
        attrs = [(str(key), str(value)) for key, value in attrs]
        attrs = set(attrs)
        values = defaultdict(set)
        for key, value in attrs:
            values[key].add(value)
        keys = [key for key in values if len(values[key]) > 1]
        return keys

    def set_data(self, data=None):
        """Set the input data table.
        """
        self.closeContext()
        self.clear()
        self.error(0)
        self.warning(0)
        if data and not self.get_suitable_keys(data):
            self.error(0, "Data has no suitable column labels.")
            data = None

        self.data = data

        if data:
            self.info_box.setText("{0} genes\n{1} experiments"
                                  .format(len(data), len(data.domain)))
            self.update_control()
            self.split_data()
        else:
            self.separate_view.setModel(itemmodels.PyListModel([]))
            self.relevant_view.setModel(itemmodels.PyListModel([]))
            self.groups_scroll_area.setWidget(QWidget())
            self.info_box.setText("No data on input.\n")
        self.commit()

    def update_control(self):
        """Update the control area of the widget. Populate the list
        views with keys from attribute labels.
        """
        keys = self.get_suitable_keys(self.data)

        model = itemmodels.PyListModel(keys)
        self.separate_view.setModel(model)
        self.separate_view.selectionModel().selectionChanged.connect(
            self.on_separate_key_changed)

        model = itemmodels.PyListModel(keys)
        self.relevant_view.setModel(model)
        self.relevant_view.selectionModel().selectionChanged.connect(
            self.on_relevant_key_changed)

        self.openContext(keys)

        # Get the selected keys from the open context
        separate_keys = self.separate_keys
        relevant_keys = self.relevant_keys

        def select(model, selection_model, selected_items):
            all_items = list(model)
            try:
                indices = [all_items.index(item) for item in selected_items]
            except:
                indices = []
            selection = QItemSelection()
            for ind in indices:
                index = model.index(ind)
                selection.select(index, index)

            selection_model.select(selection, QItemSelectionModel.Select)

        self._disable_updates = True
        try:
            select(self.relevant_view.model(),
                   self.relevant_view.selectionModel(),
                   relevant_keys)

            select(self.separate_view.model(),
                   self.separate_view.selectionModel(),
                   separate_keys)
        finally:
            self._disable_updates = False

    def on_separate_key_changed(self, *args):
        if not self._disable_updates:
            self.separate_keys = self.selected_separeate_by_keys()
            self.split_data()

    def on_relevant_key_changed(self, *args):
        if not self._disable_updates:
            self.relevant_keys = self.selected_relevant_keys()
            self.split_data()

    def selected_separeate_by_keys(self):
        """Return the currently selected separate by keys
        """
        rows = self.separate_view.selectionModel().selectedRows()
        rows = sorted([idx.row() for idx in rows])
        keys = [self.separate_view.model()[row] for row in rows]
        return keys

    def selected_relevant_keys(self):
        """Return the currently selected relevant keys
        """
        rows = self.relevant_view.selectionModel().selectedRows()
        rows = sorted([idx.row() for idx in rows])
        keys = [self.relevant_view.model()[row] for row in rows]
        return keys

    def split_data(self):
        """Split the data and update the Groups widget
        """
        separate_keys = self.selected_separeate_by_keys()
        relevant_keys = self.selected_relevant_keys()

        self.warning(0)
        if not separate_keys:
            self.warning(0, "No separate by column selected.")

        partitions, uniquepos = separate_by(
            self.data, separate_keys, consider=relevant_keys)
        partitions = partitions.items()

        all_values = defaultdict(set)
        for a in [at.attributes for at in self.data.domain.attributes]:
            for k, v in a.items():
                all_values[k].add(v)

        # sort groups
        pkeys = [key for key, _ in partitions]
        types = [data_type([a[i] for a in pkeys])
                 for i in range(len(pkeys[0]))]

        partitions = sorted(partitions, key=lambda x:
                    tuple(types[i](v) for i,v in enumerate(x[0])))

        split_groups = []

        # Collect relevant key value pairs for all columns
        relevant_items = None

        for keys, indices in partitions:
            if relevant_items == None:
                relevant_items = [defaultdict(set) for _ in indices]
            for i, ind in enumerate(indices):
                if ind is not None:
                    attr = self.data.domain[ind]
                    for key in relevant_keys:
                        relevant_items[i][key].add(attr.attributes[key])

        #those with different values between rows are not relevant
        for d in relevant_items:
            for k, s in list(d.items()):
                if len(s) > 1:
                    del d[k]
                else:
                    d[k] = s.pop()

        def get_attr(attr_index, i):
            if attr_index is None:
                attr = Orange.data.ContinuousVariable(next(missing_name_gen), 
                    compute_value=lambda x: None)
                attr.attributes.update(relevant_items[i])
                return attr
            else:
                return self.data.domain[attr_index]

        for keys, indices in partitions:
            attrs = [get_attr(attr_index, i)
                     for i, attr_index in enumerate(indices)]
            for attr in attrs:
                attr.attributes.update(zip(separate_keys, keys))
            domain = Orange.data.Domain(attrs, [], self.data.domain.metas)
            split_groups.append((keys, domain))

        self.set_groups(separate_keys, split_groups, relevant_keys,
                        relevant_items, all_values, uniquepos)

        self.partitions = partitions
        self.split_groups = split_groups

        self.commit()

    def set_groups(self, keys, groups, relevant_keys, relevant_items,
                   all_values, uniquepos):
        """Set the current data groups and update the Group widget
        """
        layout = QVBoxLayout()
        header_widths = []
        header_views = []
        palette = self.palette()
        all_values = all_values.keys()

        def for_print(rd):
            attrs = []
            for d in rd:
                attr = Orange.data.ContinuousVariable(next(inactive_name_gen))
                attr.attributes.update(d)
                attrs.append(attr)
            return Orange.data.Domain(attrs, None)

        for separatev, domain in [(None, for_print(relevant_items))] + groups:
            label = None
            if separatev is not None:
                ann_vals = " <b>|</b> ".join(["<b>{0}</b> = {1}".format(key,val) \
                     for key, val in zip(keys, separatev)])
                label = QLabel(ann_vals)

            model = QStandardItemModel()
            for i, attr in enumerate(domain.attributes):
                item = QStandardItem()
                if separatev is not None:
                    isunique = uniquepos[separatev][i]
                else:
                    isunique = all(a[i] for a in uniquepos.values())

                if str(attr.name).startswith("!!missing "):  # TODO: Change this to not depend on name
                    header_text = ["{0}={1}".format(key, attr.attributes.get(key, "?")) \
                                   for key in all_values if key not in relevant_items[i]]
                    header_text = "\n".join(header_text) if header_text else "Empty"
                    item.setData(header_text, Qt.DisplayRole)
                    item.setFlags(Qt.NoItemFlags)
                    item.setData(QColor(Qt.red), Qt.ForegroundRole)
                    item.setData(palette.color(QPalette.Disabled, QPalette.Window), Qt.BackgroundRole)
                    item.setData("Missing feature.", Qt.ToolTipRole)
                elif str(attr.name).startswith("!!inactive "):
                    header_text = ["{0}={1}".format(key, attr.attributes.get(key, "?")) \
                                   for key in all_values if key in relevant_items[i]]
                    header_text = "\n".join(header_text) if header_text else "No descriptor"
                    item.setData(header_text, Qt.DisplayRole)
                    item.setData(palette.color(QPalette.Disabled, QPalette.Window), Qt.BackgroundRole)
                else:
                    header_text = ["{0}={1}".format(key, attr.attributes.get(key, "?")) \
                                   for key in all_values if key not in relevant_items[i]]
                    header_text = "\n".join(header_text) if header_text else "Empty"
                    item.setData(header_text, Qt.DisplayRole)
                    item.setData(attr.name, Qt.ToolTipRole)

                if not isunique:
                    item.setData(QColor(Qt.red), Qt.ForegroundRole)

                model.setHorizontalHeaderItem(i, item)
            attr_count = len(domain.attributes)
            view = MyHeaderView(Qt.Horizontal)
            view.setResizeMode(QHeaderView.Fixed)
            view.setModel(model)
            hint = view.sizeHint()
            view.setMaximumHeight(hint.height())

            widths = [view.sectionSizeHint(i) for i in range(attr_count)]
            header_widths.append(widths)
            header_views.append(view)

            if label:
                layout.addWidget(label)
            layout.addWidget(view)
            layout.addSpacing(8)

        # Make all header sections the same width
        width_sum = 0
        max_header_count = max([h.count() for h in header_views])
        for i in range(max_header_count):
            max_width = max([w[i] for w in header_widths if i < len(w)] or [0])
            for view in header_views:
                if i < view.count():
                    view.resizeSection(i, max_width)
            width_sum += max_width + 2

        for h in header_views:
            h.setMinimumWidth(h.length() + 4)

        widget = QWidget()
        widget.setLayout(layout)
        widget.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Maximum)
        layout.activate()

        max_width = max(h.length() for h in header_views) + 20

        left, _, right, _ = self.getContentsMargins()
        widget.setMinimumWidth(width_sum)
        widget.setMinimumWidth(max_width + left + right)
        self.groups_scroll_area.setWidget(widget)

    def compute_distances(self, separate_keys, partitions, data):
        """Compute the distances between genotypes.
        """
        if separate_keys and partitions:
            self.progressBarInit()
#             matrix = Orange.misc.DistMatrix(len(partitions))
            matrix = numpy.zeros((len(partitions), len(partitions)))

            profiles = [linearize(data, indices) for _, indices in partitions]
            dist_func = self.DISTANCE_FUNCTIONS[self.distance_measure][1]
#             from Orange.utils import progress_bar_milestones
            count = (len(profiles) * len(profiles) - 1) / 2
#             milestones = progress_bar_milestones(count)
            iter_count = 0
            for i in range(len(profiles)):
                for j in range(i + 1, len(profiles)):
                    matrix[i, j] = dist_func(profiles[i], profiles[j])
                    matrix[j, i] = matrix[i, j]
                    iter_count += 1
#                     if iter_count in milestones:
                    self.progressBarSet(100.0 * iter_count / count)
            self.progressBarFinished()

            items = [["{0}={1}".format(key, value)
                      for key, value in zip(separate_keys, values)]
                     for values, _ in partitions]
            items = [" | ".join(item) for item in items]
#             matrix.setattr("items", items)
            matrix = Orange.misc.DistMatrix(matrix)
        else:
            matrix = None

        self.matrix = matrix

    def commit(self):
        separate_keys = self.selected_separeate_by_keys()
        self.compute_distances(separate_keys,
                               self.partitions,
                               self.data)
        if self.split_groups:
            all_attrs = []
            for group, domain in self.split_groups:
                attrs = []
                group_name = " | ".join("{0}={1}".format(*item) for item in
                                        zip(separate_keys, group))
                for attr in domain.attributes:
                    newattr = clone_attr(attr)
                    newattr.attributes["<GENOTYPE GROUP>"] = group_name # Need a better way to pass the groups to downstream widgets.
                    attrs.append(newattr)

                all_attrs.extend(attrs)

            domain = Orange.data.Domain(all_attrs, self.data.domain.class_vars,
                                        self.data.domain.metas)
            data = Orange.data.Table(domain, self.data)
        else:
            data = None
        self.send("Sorted Data", data)
        self.send("Distances", self.matrix)

    def send_report(self):
        self.report_items((
            ("Separate By", ", ".join(self.selected_separeate_by_keys())),
            ("Sort By", ", ".join(self.selected_relevant_keys())),
            ("Distance Measure", self.DISTANCE_FUNCTIONS[self.distance_measure][0])
        ))
        layout = self.groups_scroll_area.widget().layout()
        html = "<table>"
        for i in range(layout.count()):
            item = layout.itemAt(i)
            if isinstance(item, QSpacerItem):
                html += "<tr><td></td></tr>"
            elif isinstance(item, QWidgetItem):
                hor = item.widget()
                if isinstance(hor, QLabel):
                    label = hor.text()
                    html += "<tr><td><b>%s</b></td></tr>" % label
                elif isinstance(hor, QHeaderView):
                    model = hor.model()
                    content = (model.horizontalHeaderItem(col) for col in range(model.columnCount()))
                    content = (item.text().replace('\n', "<br/>") for item in content)
                    html += "<tr>" + ''.join("<td>{}</td>".format(item) for item in content) + "</tr>"
        html += "</table>"
        self.report_raw("Groups", html)


def test_main(argv=sys.argv):
    import os
    from AnyQt.QtWidgets import QApplication

    if len(argv) > 1:
        filename = argv[1]
    else:
        filename = os.path.expanduser("~/GDS624.tab")
    data = Orange.data.Table(filename)

    app = QApplication(sys.argv)
    w = OWGenotypeDistances()
    w.set_data(data)
    w.show()
    w.raise_()
    r = app.exec_()
    w.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())



"""
Table and other data manipulation utilities
"""
import numbers
from operator import itemgetter
from typing import Sequence, Tuple

import numpy

import Orange.data

ColSpec = Sequence[Tuple[Orange.data.Variable, Sequence[numbers.Real]]]


def append_columns(data, attributes=(), class_vars=(), metas=()):
    # type: (Orange.data.Table, ColSpec, ColSpec, ColSpec) -> Orange.data.Table
    """
    Append a set of columns to a data table.


    Parameters
    ----------
    data : Orange.data.Table
        Primary table.
    attributes : Sequence[Tuple[Orange.data.Variable], Sequence[float]]
        A Sequence of variable and column data tuples to append to the
        `data`.
    class_vars : Sequence[Tuple[Orange.data.Variable], Sequence[float]]
        A Sequence of variable and column data tuples to append to the
        `data` in the
    metas : Sequence[Tuple[Orange.data.Variable], Sequence[float]]
        A Sequence of variable and column data tuples to append to the
        `data`

    Returns
    -------
    data : Orange.data.Table
        A copy of the original `data` input extended with all columns from
        `attributes`, `class_vars`, `metas` parameters

    Note
    ----
    All variables in the original and new columns should be distinct.
    """
    domain = data.domain
    new_attributes = tuple(map(itemgetter(0), attributes))
    new_class_vars = tuple(map(itemgetter(0), class_vars))
    new_metas = tuple(map(itemgetter(0), metas))

    new_domain = Orange.data.Domain(
        domain.attributes + new_attributes,
        domain.class_vars + new_class_vars,
        domain.metas + new_metas
    )

    def ascolumn(array, n):
        # type: (Sequence[float], int) -> numpy.ndarray
        array = numpy.asarray(array)
        if array.ndim < 2:
            array = array.reshape((n, 1))
        return array
    N = len(data)

    attr_cols = [ascolumn(col, N) for _, col in attributes]
    class_cols = [ascolumn(col, N) for _, col in class_vars]
    meta_cols = [ascolumn(col, N) for _, col in metas]

    new_data = data.from_table(new_domain, data)

    for i, (var, col) in enumerate(zip(new_attributes, attr_cols),
                                   start=len(domain.attributes)):
        assert new_data.domain.attributes[i] is var
        new_data.X[:, i] = col.ravel()

    for i, (var, col) in enumerate(zip(new_class_vars, class_cols),
                                   start=len(domain.class_vars)):
        assert new_data.domain.class_vars[i] is var
        new_data._Y[:, i] = col.ravel()

    for i, (var, col) in enumerate(zip(new_metas, meta_cols),
                                   start=len(domain.metas)):
        assert new_data.domain.metas[i] is var
        new_data.metas[:, i] = col.ravel()

    return new_data


import unittest


class TestDataUtils(unittest.TestCase):
    def test_append_columns(self):
        data = Orange.data.Table.from_numpy(
            None,
            numpy.array([[1, 2], [2, 3]]),
            numpy.array([[1], [2]]),
            # numpy.array([[], []])
        )
        dv = Orange.data.DiscreteVariable("D", values=("A", "B"))
        cv = Orange.data.ContinuousVariable("C")
        r = append_columns(data, attributes=[(dv, [0, 1])])
        self.assertEqual(r.domain.attributes, data.domain.attributes + (dv,))
        self.assertEqual(r.domain.class_vars, data.domain.class_vars)
        self.assertEqual(r.domain.metas, data.domain.metas)
        numpy.testing.assert_equal(r.X[:, -1], [0, 1])
        numpy.testing.assert_equal(r.X[:, :-1], data.X)
        numpy.testing.assert_equal(r.Y, data.Y)
        numpy.testing.assert_equal(r.metas, data.metas)

        r = append_columns(data, class_vars=[(cv, [.0, .5])])
        self.assertEqual(r.domain.attributes, data.domain.attributes)
        self.assertEqual(r.domain.class_vars, data.domain.class_vars + (cv,))
        self.assertEqual(r.domain.metas, data.domain.metas)
        numpy.testing.assert_equal(r.X, data.X)
        numpy.testing.assert_equal(r._Y[:, :-1], data._Y)
        numpy.testing.assert_equal(r._Y[:, -1], [.0, .5])
        numpy.testing.assert_equal(r.metas, data.metas)

        r = append_columns(data, metas=[(dv, [0, 1])])
        self.assertEqual(r.domain.attributes, data.domain.attributes)
        self.assertEqual(r.domain.class_vars, data.domain.class_vars)
        self.assertEqual(r.domain.metas, data.domain.metas + (dv,))
        numpy.testing.assert_equal(r.X, data.X)
        numpy.testing.assert_equal(r._Y, r._Y)
        numpy.testing.assert_equal(r.metas[:, :-1], data.metas)
        numpy.testing.assert_equal(r.metas[:, -1], [0, 1])

        data_0 = data[:0]
        r = append_columns(data_0, attributes=[(cv, [])], metas=[(dv, [])])
        self.assertEqual(r.domain.attributes, data.domain.attributes + (cv,))
        self.assertEqual(r.domain.metas, data.domain.metas + (dv,))



"""Tools"""
from .. import dicty
from Orange.data import ContinuousVariable, StringVariable, TimeVariable, Domain, Table


CallBack = dicty.CallBack
transformValues = dicty.transformValues
averageAttributes = dicty.averageAttributes
example_tables = dicty.example_tables


def transpose_table(table):
    """
    Transpose the rows and columns of the table.

    Args:
        table: Data in :obj:`Orange.data.Table`

    Returns:
         Transposed :obj:`Orange.data.Table`. (Genes as columns)
    """
    attrs = table.domain.attributes
    attr = [ContinuousVariable.make(ex['Gene'].value) for ex in table]
    #  Set metas
    new_metas = [StringVariable.make(name) if name is not 'Time' else TimeVariable.make(name)
                 for name in sorted(table.domain.variables[0].attributes.keys())]
    domain = Domain(attr, metas=new_metas)
    meta_values = [[exp.attributes[var.name] for var in domain.metas] for exp in attrs]

    return Table(domain, table.X.transpose(), metas=meta_values)



import itertools
from collections import defaultdict, namedtuple
from xml.sax.saxutils import escape

import numpy

from AnyQt import QtGui
from AnyQt.QtCore import Qt

import Orange.data
from Orange.widgets import gui

from . import gui as guiutils

ColumnGroup = namedtuple("ColumnGroup", ["name", "key", "values"])
RowGroup = namedtuple("RowGroup", ["name", "var", "values"])


def group_candidates(data):
    items = [attr.attributes.items() for attr in data.domain.attributes]
    items = list(itertools.chain(*items))

    targets = defaultdict(set)
    for label, value in items:
        targets[label].add(value)

    # Need at least 2 distinct values or key
    targets = [(key, sorted(vals)) for key, vals in targets.items() \
               if len(vals) >= 2]

    column_groups = [ColumnGroup(key, key, values)
                     for key, values in sorted(targets)]
    disc_vars = [var for var in data.domain.class_vars + data.domain.metas
                 if isinstance(var, Orange.data.DiscreteVariable)
                 and len(var.values) >= 2]

    row_groups = [RowGroup(var.name, var, var.values)
                  for var in disc_vars]
    return column_groups, row_groups


@guiutils.standarditem_from.register(ColumnGroup)
def standarditem_from_columngroup(colgroup):
    item = QtGui.QStandardItem(colgroup.name)
#     item.setIcon(pkg_path('columnset.svg'))
    item.setToolTip("Split by column label: '{!s}'"
                    .format(escape(colgroup.name)))
    item.setFlags(item.flags() & ~Qt.ItemIsEditable)
    item.setData(colgroup, Qt.UserRole)
    children = [guiutils.standarditem_from(val)
                for val in colgroup.values]
    item.appendRows(children)
    return item


@guiutils.standarditem_from.register(RowGroup)
def standarditem_from_rowgroup(rowgroup):
    item = QtGui.QStandardItem(rowgroup.name)
    icon, _ = gui.attributeItem(rowgroup.var)
    item.setIcon(icon)
    item.setToolTip(guiutils.variable_tooltip(rowgroup.var))
    item.setData(rowgroup, Qt.UserRole)
    item.setFlags(item.flags() & ~Qt.ItemIsEditable)
    children = [guiutils.standarditem_from(val)
                for val in rowgroup.values]
    item.appendRows(children)
    return item


def group_selection_mask(data, group, indices):
    """
    Return the selection masks for the group.
    """
    if isinstance(group, ColumnGroup):
        selected = [group.values[i] for i in indices]
        target = set([(group.key, value) for value in selected])
        I = [bool(set(var.attributes.items()).intersection(target))
             for var in data.domain.attributes]
        return numpy.array(I, dtype=bool)
    elif isinstance(group, RowGroup):
        target = set(indices)
        X, _ = data.get_column_view(group.var)
        I = numpy.zeros_like(X, dtype=bool)
        for i in target:
            I |= X == i
        return I
    else:
        raise TypeError("ColumnGroup or RowGroup expected, got {}"
                        .format(type(group).__name__))



import sys
import os

from functools import partial, reduce

import numpy
from AnyQt.QtGui import QPainter
from AnyQt.QtCore import QObject, QSize, QThread, QThreadPool, Slot

import pyqtgraph as pg

import Orange.data

from Orange.widgets import widget, gui, settings
from Orange.widgets.utils import concurrent

from ..utils import expression


def is_discrete(var):
    return isinstance(var, Orange.data.DiscreteVariable)


def group_mask_columns(data, key, values):
    """
    Return a boolean array mask of data columns (variables).

    The mask will be True wherever the `var.attributes[key]` contains
    one of `values`.

    Parameters
    ----------
    data : Orange.data.Table
        Source data table.
    key : str
        The variable label key (where `values` are looked for)
    values : sequence of str
        The values (for the corresponding `key`) selecting the columns.
    """
    target = set([(key, value) for value in values])
    mask = [not target.isdisjoint(var.attributes.items())
            for var in data.domain.attributes]
    return numpy.array(mask, dtype=bool)


def group_mask_rows(data, var, values):
    """
    Return a boolean array mask for data rows (instances).

    The mask will be True wherever the row's entry for `var` contains
    one of the `values`.

    Parameters
    ----------
    data : Orange.data.Table
        Source data table.
    var : Orange.data.DiscreteVariable
        The variable/column on which to match `values`.
    values : sequence of str
        The values to select (must be a subset of `var.values`)
    """
    var = data.domain[var]
    col_view, _ = data.get_column_view(var)
    target_ind = [var.values.index(t) for t in values]

    mask = numpy.zeros_like(col_view, dtype=bool)
    for i in target_ind:
        mask |= col_view == i

    return mask


def group_mask(data, key, values, axis=1):
    if axis == 1:
        return group_mask_columns(data, key, values)
    elif axis == 0:
        return group_mask_rows(data, key, values)
    else:
        raise ValueError("0 <= axis < 2")


def table_take(data, indices, axis=0):
    if axis == 0:
        return data[indices]
    elif axis == 1:
        return data[:, indices]


class ScatterPlotItem(pg.ScatterPlotItem):
    def paint(self, painter, option, widget):
        if self.opts["antialias"]:
            painter.setRenderHint(QPainter.Antialiasing, True)
        if self.opts["pxMode"]:
            painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
        super().paint(painter, option, widget)


class ProgressBarDiscard(QObject):

    def __init__(self, parent, redirect):
        QObject.__init__(self, parent)
        self.redirect = redirect
        self._delay = False

    @Slot(float)
    def progressBarSet(self, value):
        # Discard OWBaseWidget.progressBarSet call, because it calls qApp.processEvents
        # which can result in 'event queue climbing' and max. recursion error if GUI thread
        # gets another advance signal before it finishes with this one
        if not self._delay:
            try:
                self._delay = True
                self.redirect.progressBarSet(value)
            finally:
                self._delay = False


def withexcepthook(func):
    def wrapped(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except BaseException as ex:
            sys.excepthook(*sys.exc_info())
            raise

    return wrapped


class OWMAPlot(widget.OWWidget):
    name = "MA Plot"
    description = "Normalize expression array data on a MA - plot"
    icon = "../widgets/icons/MAPlot.svg"
    priority = 5000

    inputs = [("Expression array", Orange.data.Table, "setData")]
    outputs = [("Normalized expression array", Orange.data.Table,
                widget.Default),
               ("Filtered expression array", Orange.data.Table)]

    CENTER_METHODS = [("Average", expression.MA_center_average),
                      ("Lowess (fast - interpolated)", expression.MA_center_lowess_fast),
                      ("Lowess", expression.MA_center_lowess)]

    MERGE_METHODS = [("Average", numpy.ma.average),
                     ("Median", numpy.ma.median),
                     ("Geometric mean", expression.geometric_mean)]

    settingsHandler = settings.DomainContextHandler()

    appendZScore = settings.Setting(False)
    appendRIValues = settings.Setting(False)

    selectedGroup = settings.ContextSetting(0)
    selectedCenterMethod = settings.Setting(0)
    selectedMergeMethod = settings.Setting(0)
    zCutoff = settings.Setting(1.96)

    autoCommit = settings.Setting(False)

    def __init__(self, parent=None):
        super().__init__(parent)

        # GUI
        box = gui.widgetBox(self.controlArea, "Info", addSpace=True)
        self.infoBox = gui.widgetLabel(box, "No data on input.")

        box = gui.widgetBox(self.controlArea, "Split by", addSpace=True)
        self.groupCombo = gui.comboBox(
            box, self, "selectedGroup", callback=self.onGroupSelection)

        gui.comboBox(self.controlArea, self, "selectedCenterMethod",
                     box="Center Fold-change Using",
                     items=[name for name, _ in self.CENTER_METHODS],
                     callback=self.onCenterMethodChange,
                     addSpace=True)

        gui.comboBox(self.controlArea, self, "selectedMergeMethod",
                     box="Merge Replicates",
                     items=[name for name, _ in self.MERGE_METHODS],
                     tooltip="Select the method for replicate merging",
                     callback=self.onMergeMethodChange,
                     addSpace=True)

        box = gui.doubleSpin(self.controlArea, self, "zCutoff", 0.0, 3.0, 0.01,
                             box="Z-Score Cutoff",
                             callback=[self.replotMA, self.commitIf])

        gui.separator(self.controlArea)

        box = gui.widgetBox(self.controlArea, "Ouput")
        gui.checkBox(box, self, "appendZScore", "Append Z-Scores",
                     tooltip="Append calculated Z-Scores to output",
                     callback=self.commitIf)

        gui.checkBox(box, self, "appendRIValues",
                     "Append Log Ratio and Intensity values",
                     tooltip="Append calculated Log Ratio and Intensity "
                             "values to output data",
                     callback=self.commitIf)

        gui.rubber(self.controlArea)

        gui.auto_commit(self.controlArea, self, "autoCommit", "Commit")

        self.graph = pg.PlotWidget(background="w")
        self.graph.getAxis("bottom").setLabel("Intensity: log<sub>10</sub>(R*G)")
        self.graph.getAxis("left").setLabel("Log ratio: log<sub>2</sub>(R/G)")

        self.mainArea.layout().addWidget(self.graph)
        self.groups = []
        self.split_data = None, None
        self.merged_splits = None, None
        self.centered = None, None
        self.changedFlag = False
        self.data = None

        self._executor = concurrent.ThreadExecutor(
            threadPool=QThreadPool(maxThreadCount=1))

    def sizeHint(self):
        return QSize(800, 600)

    def onGroupSelection(self):
        if self.data:
            self.updateInfoBox()
            self.splitData()
            self.runNormalization()

    def onCenterMethodChange(self):
        if self.data:
            self.runNormalization()

    def onMergeMethodChange(self):
        if self.data:
            self.splitData()
            self.runNormalization()

    def proposeGroups(self, data):
        col_labels = [attr.attributes.items()
                      for attr in data.domain.attributes]
        col_labels = sorted(reduce(set.union, col_labels, set()))
        col_labels = [(key, value, 1) for key, value in col_labels]

        attrs = [attr for attr in data.domain.variables + data.domain.metas
                 if is_discrete(attr)]

        row_labels = [(attr.name, value, 0)
                      for attr in attrs for value in attr.values]

        def filterSingleValues(labels):
            ret = []
            for name, value, axis in labels:
                match = [(n, v, a) for n, v, a in labels if n == name]
                if len(match) > 1:
                    ret.append((name, value, axis))
            return ret

        col_labels = filterSingleValues(col_labels)
        row_labels = filterSingleValues(row_labels)

        return col_labels + row_labels

    def setData(self, data):
        self.closeContext()
        self.clear()
        self.unconditional_commit()
        self.data = data
        self.error([0, 1])
        if data is not None:
            self.infoBox.setText("%i genes on input" % len(data))
            self.groups = self.proposeGroups(data)
            self.groupCombo.clear()
            self.groupCombo.addItems(["%s: %s" % (key, value)
                                      for key, value, axis in self.groups])

            if not self.groups:
                self.error(1, "Input data has no class attribute or attribute labels!")
                self.clear()
                return

            self.openContext(data)
            self.selectedGroup = min(self.selectedGroup, len(self.groups) - 1)

            self.updateInfoBox()
            self.splitData()
            self.runNormalization()
        else:
            self.clear()

    def clear(self):
        self.error([0, 1])
        self.data = None
        self.groups = []
        self.centered = None, None
        self.split_data = None, None
        self.merged_splits = None, None
        self.groupCombo.clear()

        self.graph.clear()
        self.infoBox.setText("No data on input")

    def updateInfoBox(self):
        # TODO: report on number of columns or rows depending
        # on the genes 'axis' 
        self.infoBox.setText("%i genes on input" % len(self.data))

    def getSelectedGroup(self):
        return self.groups[self.selectedGroup]

    def getSelectedGroupSplit(self):
        key, value, axis = self.getSelectedGroup()
        other_values = [v for k, v, a in self.groups
                        if k == key and a == axis and v != value]
        return [(key, [value]), (key, other_values)], axis

    def getGeneNames(self):
        key, value, axis = self.getSelectedGroup()
        if axis == 0:
            genes = [str(ex[key]) for ex in self.data]
        else:
            genes = [attr.name for attr in self.data.domain.attributes]

        return genes

    def splitData(self):
        groups, axis = self.getSelectedGroupSplit()
        self.split_ind = [expression.select_indices(self.data, key, value, axis)
                          for key, value in groups]
        self.split_ind = [numpy.flatnonzero(
                              group_mask(self.data, key, value, axis))
                          for key, value in groups]

        self.split_data = [table_take(self.data, indices, axis)
                           for indices in self.split_ind]

    def getMerged(self):
        split1, split2 = self.split_data
        array1 = numpy.ma.array(split1.X, copy=False, mask=numpy.isnan(split1.X))
        array2 = numpy.ma.array(split2.X, copy=False, mask=numpy.isnan(split2.X))

        _, _, axis = self.getSelectedGroup()
        merge_function = self.MERGE_METHODS[self.selectedMergeMethod][1]

        merged1 = expression.merge_replicates(
            array1, axis, merge_function=merge_function)
        merged2 = expression.merge_replicates(
            array2, axis, merge_function=merge_function)
        self.merged_splits = merged1, merged2

        return self.merged_splits

    def runNormalization(self):
        self.progressBarInit()
        self.progressBarSet(0.0)
        G, R = self.getMerged()
        self.progressBarSet(5.0)

        center_method = self.CENTER_METHODS[self.selectedCenterMethod][1]

        # TODO: progess bar , lowess can take a long time
        if self.selectedCenterMethod in [1, 2]:  # Lowess
            Gc, Rc = center_method(G, R, f=1. / min(500., len(G) / 100), iter=1)
        else:
            Gc, Rc = center_method(G, R)
        self.progressBarSet(70.0)
        self.centered = Gc, Rc
        self.z_scores = expression.MA_zscore(Gc, Rc, 1. / 3.)
        self.progressBarSet(100.0)
        self.plotMA(Gc, Rc, self.z_scores, self.zCutoff)
        self.progressBarFinished()

    def runNormalizationAsync(self):
        """ Run MA centering and z_score estimation in a separate thread 
        """
        self.error(0)
        self.progressBarInit(processEvents=None)
        self.progressBarSet(0.0, processEvents=None)
        G, R = self.getMerged()

        center_method = self.CENTER_METHODS[self.selectedCenterMethod][1]
        use_lowess = self.selectedCenterMethod in [1, 2]

        @withexcepthook
        def run(progressCallback=lambda value: None):
            if use_lowess:
                Gc, Rc = center_method(
                    G, R, f=2. / 3., iter=1,
                    progressCallback=lambda val: progressCallback(val / 2))
            else:
                Gc, Rc = center_method(G, R)
            progressCallback(50)
            z_scores = expression.MA_zscore(
                Gc, Rc, 1. / 3.,
                progressCallback=lambda val: progressCallback(50 + val / 2))

            return Gc, Rc, z_scores

        self.progressDiscard = ProgressBarDiscard(self, self)

        progress = concurrent.methodinvoke(
            self.progressDiscard, "progressBarSet", (float,))

        self._task = concurrent.Task(function=partial(run, progress))
        self._task.resultReady.connect(self.onResultsReady)
        self._task.exceptionReady.connect(self.onException)

        self.setEnabled(False)
        self.setBlocking(True)

        self._executor.submit(self._task)

    # comment out this line if threading creates any problems
    runNormalization = runNormalizationAsync

    def onResultsReady(self, results):
        """Handle the results of centering and z-scoring
        """
        assert(QThread.currentThread() is self.thread())
        Gc, Rc, z_scores = results

        self.setEnabled(True)
        self.setBlocking(False)
        self.progressBarFinished()
        self.centered = Gc, Rc
        self.z_scores = z_scores
        self.plotMA(Gc, Rc, z_scores, self.zCutoff)
        self.commitIf()

    def onException(self, error):
        print("Error: ", error, file=sys.stderr)
        self.error(0, "Error: {!s}".format(error))
        self.setEnabled(True)
        self.setBlocking(False)

    def plotMA(self, G, R, z_scores, z_cuttof):
        ratio, intensity = expression.ratio_intensity(G, R)

        validmask = (numpy.isfinite(ratio) &
                     numpy.isfinite(intensity) &
                     numpy.isfinite(z_scores))
        for array in [ratio, intensity, z_scores]:
            if numpy.ma.is_masked(array):
                validmask &= ~array.mask

        filtered_ind = numpy.ma.where(validmask)
        ratio = numpy.take(ratio, filtered_ind)
        intensity = numpy.take(intensity, filtered_ind)
        z_scores = numpy.take(z_scores, filtered_ind)

        red_ind = numpy.ma.where(numpy.ma.abs(z_scores) >= z_cuttof)
        blue_ind = numpy.ma.where(numpy.ma.abs(z_scores) < z_cuttof)

        red_xdata, red_ydata = intensity[red_ind], ratio[red_ind]
        blue_xdata, blue_ydata = intensity[blue_ind], ratio[blue_ind]

        self.graph.clear()
        line = pg.InfiniteLine(pos=(0, 0), angle=0, pen=pg.mkPen((0, 0, 0)))
        self.graph.addItem(line)

        red_points = ScatterPlotItem(
            x=red_xdata, y=red_ydata,
            brush=pg.mkBrush((255, 0, 0, 100)), size=6, antialias=True)
        blue_points = ScatterPlotItem(
            x=blue_xdata, y=blue_ydata,
            brush=pg.mkBrush((0, 0, 255, 100)), size=6, antialias=True)

        self.graph.addItem(red_points)
        self.graph.addItem(blue_points)

    def replotMA(self):
        if self.data and self.centered:
            Gc, Rc = self.centered
            self.plotMA(Gc, Rc, self.z_scores, self.zCutoff)

    def commitIf(self):
        self.commit()

    def commit(self):
        if not self.data:
            self.send("Normalized expression array", None)
            self.send("Filtered expression array", None)
            return

        G, R = self.merged_splits
        Gc, Rc = self.centered
        ind1, ind2 = self.split_ind

        gfactor = Gc / G
        domain = self.data.domain
        newmetas = []
        M = []

        _, _, axis = self.getSelectedGroup()
        if self.appendZScore and axis == 1:
            attr = Orange.data.ContinuousVariable("Z-Score")
            newmetas.append(attr)
            M.append(self.z_scores.filled(numpy.nan))

        if self.appendRIValues and axis == 1:
            r_attr = Orange.data.ContinuousVariable("Log Ratio")
            i_attr = Orange.data.ContinuousVariable("Intensity")
            ratio, intensity = expression.ratio_intensity(Gc, Rc)
            newmetas.extend([r_attr, i_attr])
            M.extend([ratio.filled(numpy.nan),
                      intensity.filled(numpy.nan)])

        if newmetas:
            domain = Orange.data.Domain(
                self.data.domain.attributes, self.data.domain.class_vars,
                self.data.domain.metas + tuple(newmetas))

        data = Orange.data.Table.from_table(domain, self.data)
        data.ensure_copy()

        if axis == 0:
            data.X[ind1, :] *= gfactor.reshape((1, -1))
        else:
            data.X[:, ind1] *= gfactor.reshape((-1, 1))

        for i, mcol in enumerate(reversed(M)):
            data.metas[:, -i - 1] = mcol

        selected_indices = numpy.flatnonzero(
            numpy.abs(self.z_scores.filled(0)) >= self.zCutoff)

        if axis == 0:
            attrs = [data.domain[i] for i in selected_indices]
            domain = Orange.data.Domain(attrs, data.domain.class_vars,
                                        data.domain.metas)
            filtered_data = data.from_table(domain, data)
        else:
            filtered_data = data[selected_indices]

        self.send("Normalized expression array", data)
        self.send("Filtered expression array", filtered_data)


def test_main(argv=sys.argv):
    from AnyQt.QtWidgets import QApplication
    if len(argv) > 1:
        filename = argv[1]
    else:
        filename = os.path.expanduser("~/GDS1210.tab")
    data = Orange.data.Table(filename)

    app = QApplication(argv)
    w = OWMAPlot()
    w.setData(data)
    w.show()
    w.raise_()
    r = app.exec_()
    w.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())



import os
import unittest


def suite(loader=None, pattern='test_*.py'):
    """
    Return the default test suite.
    """
    test_dir = os.path.dirname(__file__)
    kegg_dir = os.path.normpath(os.path.join(test_dir, "..", "kegg"))
    if loader is None:
        loader = unittest.TestLoader()
    if pattern is None:
        pattern = 'test_*.py'

    thisdir = os.path.dirname(__file__)

    top_level_dir = os.path.join(thisdir, "..", "..", "..")
    top_level_dir = os.path.realpath(top_level_dir)
    all_tests = [
        loader.discover(test_dir, pattern, top_level_dir),
        loader.discover(kegg_dir, pattern, top_level_dir),
    ]

    return unittest.TestSuite(all_tests)


def load_tests(loader, tests, pattern):
    return suite(loader, pattern)


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



import sys
import warnings
import re

from xml.sax.saxutils import escape
from contextlib import contextmanager
from itertools import starmap, chain
from collections import namedtuple, defaultdict

if sys.version_info < (3, 4):
    from singledispatch import singledispatch
else:
    from functools import singledispatch

from AnyQt import QtGui, QtCore, QtWidgets

from AnyQt.QtWidgets import QCompleter
from AnyQt.QtCore import Qt, QObject, QStringListModel, pyqtSignal as Signal

import Orange.data
from Orange.widgets.utils import itemmodels
from Orange.widgets import gui as _gui


@contextmanager
def blocked_signals(qobj):
    """
    Context manager blocking the `qobj` signals.
    """
    state = qobj.signalsBlocked()
    qobj.blockSignals(True)
    try:
        yield qobj
    finally:
        qobj.blockSignals(state)


@singledispatch
def standarditem_from(obj):
    item = QtGui.QStandardItem(str(obj))
    item.setData(obj, Qt.UserRole)
    item.setFlags(item.flags() & ~Qt.ItemIsEditable)
    return item


@standarditem_from.register(Orange.data.Variable)
def standarditem_from_var(var):
    item = QtGui.QStandardItem(var.name)
    _, icon = _gui.attributeItem(var)
    item.setIcon(icon)
    item.setToolTip(itemmodels.Variable)


def variable_tooltip(var):
    if isinstance(var, Orange.data.DiscreteVariable):
        return discrete_variable_tooltip(var)
    elif isinstance(Orange.data.ContinuousVariable):
        return continuous_variable_toltip(var)
    elif isinstance(var, Orange.data.StringVariable):
        return string_variable_tooltip(var)


def variable_labels_tooltip(var):
    text = ""
    if var.attributes:
        items = [(escape(key), escape(value))
                 for key, value in var.attributes.items()]
        labels = list(starmap("{!s} = {!s}".format, items))
        text += "<br/>Variable Labels:<br/>"
        text += "<br/>".join(labels)
    return text


def discrete_variable_tooltip(var):
    text = "<b>%s</b><br/>Discrete with %i values: " %\
           (escape(var.name), len(var.values))
    text += ", ".join("%r" % escape(v) for v in var.values)
    text += variable_labels_tooltip(var)
    return text


def continuous_variable_toltip(var):
    text = "<b>%s</b><br/>Continuous" % escape(var.name)
    text += variable_labels_tooltip(var)
    return text


def string_variable_tooltip(var):
    text = "<b>%s</b><br/>String" % escape(var.name)
    text += variable_labels_tooltip(var)
    return text


def python_variable_tooltip(var):
    text = "<b>%s</b><br/>Python" % escape(var.name)
    text += variable_labels_tooltip(var)
    return text


def group_model(rowgroups, columngroups):
    model = QtGui.QStandardItemModel()
    for key, values in rowgroups:
        item = standarditem_from(key)
        values = [standarditem_from(value) for value in values]
        item.appendRows(values)
        model.addRow(item)


class LabelSelectionWidget(QtWidgets.QWidget):
    """
    A widget for selection of label values.

    The widget displays the contents of the model with two widgets:

    * The top level model items are displayed in a combo box.
    * The children of the current selected top level item are
      displayed in a subordinate list view.

    .. note:: This is not a QAbstractItemView subclass.
    """
    #: Current group/root index has changed.
    groupChanged = Signal(int)
    #: Selection for the current group/root has changed.
    groupSelectionChanged = Signal()

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.__model = None
        self.__selectionMode = QtWidgets.QListView.ExtendedSelection

        self.__currentIndex = -1
        self.__selections = {}

        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        def group_box(title):
            box = QtWidgets.QGroupBox(title)
            box.setFlat(True)
            lay = QtWidgets.QVBoxLayout()
            lay.setContentsMargins(0, 0, 0, 0)
            box.setLayout(lay)
            return box

        self.labels_combo = QtWidgets.QComboBox()
        self.values_view = QtWidgets.QListView(
            selectionMode=self.__selectionMode
        )

        self.labels_combo.currentIndexChanged.connect(
            self.__onCurrentIndexChanged)

        l_box = group_box(self.tr("Label"))
        v_box = group_box(self.tr("Values"))

        l_box.layout().addWidget(self.labels_combo)
        v_box.layout().addWidget(self.values_view)

        layout.addWidget(l_box)
        layout.addWidget(v_box)

        self.setLayout(layout)

        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
                           QtWidgets.QSizePolicy.Expanding)

    def clear(self):
        """
        Clear the widget/model (same as ``setModel(None)``).
        """
        if self.__model is not None:
            self.values_view.selectionModel().clearSelection()
            self.values_view.selectionModel().selectionChanged.disconnect(
                self.__onSelectionChanged)

            self.values_view.setModel(None)
            self.labels_combo.setModel(
                QtGui.QStandardItemModel(self.labels_combo))
            self.__currentIndex = -1
            self.__selections = {}
            self.__model = None

    def setModel(self, model):
        """
        Set the source model for display.

        The model should be a tree model with depth 2.

        Parameters
        ----------
        model : QtCore.QAbstractItemModel
        """
        if model is self.__model:
            return

        self.clear()

        if model is None:
            return

        self.__model = model

        self.values_view.setModel(model)
        self.values_view.setRootIndex(model.index(0, 0))
        self.values_view.selectionModel().selectionChanged.connect(
            self.__onSelectionChanged)

        # will emit the currentIndexChanged (if the model is not empty)
        self.labels_combo.setModel(model)

    def model(self):
        """
        Return the current model.

        Returns
        -------
        model : QtCore.QAbstractItemModel
        """
        return self.__model

    def setCurrentGroupIndex(self, index):
        """
        Set the current selected group/root index row.

        Parameters
        ----------
        index : int
            Group index.
        """
        self.labels_combo.setCurrentIndex(index)

    def currentGroupIndex(self):
        """
        Return the current selected group/root index row.

        Returns
        -------
        row : index
            Current group row index (-1 if there is no current index)
        """
        return self.labels_combo.currentIndex()

    def setSelection(self, selection):
        """
        Set the model item selection.

        Parameters
        ----------
        selection : QtCore.QItemSelection
            Item selection.
        """
        if self.values_view.selectionModel() is not None:
            indices = selection.indexes()
            pind = defaultdict(list)

            for index in indices:
                parent = index.parent()
                if parent.isValid():
                    if parent == self.__model.index(parent.row(),
                                                    parent.column()):
                        pind[parent.row()].append(
                            QtCore.QPersistentModelIndex(index))
                    else:
                        warnings.warn("Die Die Die")
                else:
                    # top level index
                    pass

            self.__selections = pind
            self.__restoreSelection()

    def selection(self):
        """
        Return the item selection.

        Returns
        -------
        selection : QtCore.QItemSelection
        """
        selection = QtCore.QItemSelection()
        if self.__model is None:
            return selection

        for pind in chain(*self.__selections.values()):
            ind = self.__model.index(pind.row(), pind.column(), pind.parent())
            if ind.isValid():
                selection.select(ind, ind)
        return selection

    def currentGroupSelection(self):
        """
        Return the item selection for the current group only.
        """
        if self.values_view.selectionModel() is not None:
            return self.values_view.selectionModel().selection()
        else:
            return QtCore.QItemSelection()

    def __onCurrentIndexChanged(self, index):
        self.__storeSelection(self.__currentIndex,
                              self.values_view.selectedIndexes())

        self.__currentIndex = index
        if self.__model is not None:
            root = self.__model.index(index, 0)
            self.values_view.setRootIndex(root)

            self.__restoreSelection()
        self.groupChanged.emit(index)

    def __onSelectionChanged(self, old, new):
        self.__storeSelection(self.__currentIndex,
                              self.values_view.selectedIndexes())

        self.groupSelectionChanged.emit()

    def __storeSelection(self, groupind, indices):
        # Store current values selection for the current group
        groupind = self.__currentIndex
        indices = [QtCore.QPersistentModelIndex(ind)
                   for ind in self.values_view.selectedIndexes()]
        self.__selections[groupind] = indices

    def __restoreSelection(self):
        # Restore previous selection for root (if available)
        assert self.__model is not None
        groupind = self.__currentIndex
        root = self.__model.index(groupind, 0)
        sel = self.__selections.get(groupind, [])
        indices = [self.__model.index(pind.row(), pind.column(), root)
                   for pind in sel if pind.isValid() and pind.parent() == root]

        selection = QtCore.QItemSelection()
        for ind in indices:
            selection.select(ind, ind)
        self.values_view.selectionModel().select(
            selection, QtCore.QItemSelectionModel.ClearAndSelect)

    def sizeHint(self):
        """Reimplemented from QWidget.sizeHint"""
        return QtCore.QSize(100, 200)


def itemselection(modelindexlist):
    """
    Return an QtCore.QItemSelection from QModelIndex list

    Parameters
    ----------
    modelindexlist : list of QtCore.QModelIndex
        Selected model indices.

    Returns
    -------
    selection : QtCore.QItemSelection
    """
    selection = QtCore.QItemSelection()
    for index in modelindexlist:
        selection.select(index, index)
    return selection


class TokenListCompleter(QCompleter):
    """
    A completer allowing completion of multiple words (tokens)

    Example
    -------
    >>> completer = TokenListCompleter()
    >>> completer.setTokenList(["foo", "bar", "baz"])
    >>> completer.setCompletionPrefix("foo b")
    >>> for i in range(completer.completionCount()):
    ...     completer.setCurrentRow(i)
    ...     print(completer.currentCompletion())
    ...
    foo bar
    foo baz
    """
    def __init__(self, *args, **kwargs):
        separator = kwargs.pop("separator", " ")
        super().__init__(*args, **kwargs)
        self.__tokenList = []
        self.__completerModel = None
        self.__separator = separator
        # The current 'known' completion prefix (tracked in splitPath)
        self.__currentKnownPrefix = ""
        self.setModelSorting(QCompleter.CaseSensitivelySortedModel)
        super().setModel(QStringListModel(self))

    def setTokenList(self, tokenList):
        """
        Set the token word completion list

        Equivalent to
        `completer.setModel(QStringListModel(tokenList, completer))`

        Note that this is the preferred method for setting the completion
        model.

        Parameters
        ----------
        tokenList : List[str]
        """
        tokenList = list(sorted(set(tokenList)))
        self.setModel(QStringListModel(tokenList, self))

    def tokenList(self):
        """
        Return the current token word completion list.

        Return
        ------
        tokens : List[str]
        """
        return list(self.__tokenList)

    def setModel(self, model):
        """
        Reimplemented.

        Parameters
        ----------
        model : QAbstractItemModel
        """
        if model is self.__completerModel:
            return

        if self.__completerModel is not None:
            self.__completerModel.dataChanged.disconnect(
                self.__initDynamicModel)
            self.__completerModel.rowsInserted.disconnect(
                self.__initDynamicModel)
            self.__completerModel.rowsRemoved.disconnect(
                self.__initDynamicModel)

            if QObject.parent(self.__completerModel) is self:
                self.__completerModel.deleteLater()
            self.__completerModel = None

        self.__completerModel = model

        if self.__completerModel is not None:
            self.__completerModel.dataChanged.connect(
                self.__initDynamicModel)
            self.__completerModel.rowsInserted.connect(
                self.__initDynamicModel)
            self.__completerModel.rowsRemoved.connect(
                self.__initDynamicModel)

        self.__initDynamicModel()

    def model(self):
        return self.__completerModel

    def setSeparator(self, separator):
        if self.__separator != separator:
            self.__separator = separator

    def separator(self):
        return self.__separator

    # QCompleter::setCompleterPrefix is not a virtual method, and is called
    # directly by the QLineEdit to set the prefix. There does not appear to
    # be any other direct way that a QCompleter subclass could be notified
    # of the completion prefix change.
    # However in QCompleter::setCompleterPrefix
    # the prefix is immediately split by splitPath which is virtual.
    # So we use this to track changes in the current prefix.
    def splitPath(self, prefix):
        """reimplemented."""
        items = super().splitPath(prefix)
        if self.__currentKnownPrefix != self.completionPrefix():
            self.__currentKnownPrefix = self.completionPrefix()
            self.__updateEffectiveCompleterModel()
        return items

    def __updateEffectiveCompleterModel(self):
        """
        Prefix all items in the effective completer model with the current
        prefix to enable the completion of multiple keywords.
        """
        prefix = self.completionPrefix()
        model = super().model()
        assert isinstance(model, QStringListModel)
        separator = self.__separator
        if not prefix.rstrip().endswith(separator) and separator in prefix:
            # We are in the middle of a second (or later) word completion
            prefix, rest = prefix.rsplit(separator, 1)
            # preserve any whitespace after the seperator
            match = re.match(r"^(?P<whitespace>\s*)(?P<rest>.*)", rest)
            if match:
                whitespace, rest = match.groups()
            else:
                whitespace = ""
            items = [prefix + separator + whitespace + item
                     for item in self.__tokenList]
        else:
            # Either completing the the first word or immediately after
            # the separator. In both cases the original tokens list is
            # the active completion list
            items = self.__tokenList
        itemset = set(items)
        current = model.stringList()

        if itemset != set(current):
            model.setStringList(list(sorted(itemset)))

    def __initDynamicModel(self):
        """
        [Re]Initialize the effective completion model from the client
        supplied model
        """
        model = self.__completerModel
        if model is not None:
            if isinstance(model, QStringListModel):
                tokens = model.stringList()
            else:
                tokens = [model.data(model.index(row, 0), Qt.DisplayRole)
                          for row in range(model.rowCount())]
                tokens = [str(token) for token in filter(None, tokens)]
        else:
            tokens = []

        tokenList = list(sorted(set(tokens)))
        if tokenList != self.__tokenList:
            self.__tokenList = tokenList
            self.__updateEffectiveCompleterModel()

import unittest


class TestCompleter(unittest.TestCase):
    def test_TokenListCompleter(self):
        from Orange.widgets.utils.itemmodels import PyListModel
        completer = TokenListCompleter()
        completer.setTokenList(["foo", "bar", "baz"])
        completer.setCompletionPrefix("foo b")

        def completions(completer):
            current = completer.currentRow()
            items = []
            for i in range(completer.completionCount()):
                completer.setCurrentRow(i)
                items.append(completer.currentCompletion())
            completer.setCurrentRow(current)
            return items

        self.assertSequenceEqual(completions(completer),
                                 ["foo bar", "foo baz"])
        completer.setModel(None)
        self.assertSequenceEqual(completer.tokenList(), [])
        self.assertSequenceEqual(completions(completer), [])

        completer.setModel(QStringListModel(["a", "ab", "b"]))
        self.assertSequenceEqual(completer.tokenList(), ["a", "ab", "b"])
        completer.setCompletionPrefix("a a")
        self.assertSequenceEqual(completions(completer), ["a a", "a ab"])

        completer.setModel(PyListModel(["a", "aa", "ab", "ad"]))
        self.assertSequenceEqual(completer.tokenList(),
                                 ["a", "aa", "ab", "ad"])

        completer.model()[-1] = "z"
        self.assertSequenceEqual(completer.tokenList(),
                                 ["a", "aa", "ab", "z"])

        completer.model()[-2:] = ["ax", "az"]
        self.assertSequenceEqual(completer.tokenList(),
                                 ["a", "aa", "ax", "az"])

        completer.setSeparator(",")
        completer.setCompletionPrefix("a, a")
        self.assertSequenceEqual(completions(completer),
                                 ["a, a", "a, aa", "a, ax", "a, az"])



import sys
import operator
from collections import defaultdict
from contextlib import contextmanager
from functools import reduce
from xml.sax.saxutils import escape

import numpy
import scipy.stats

from AnyQt.QtWidgets import (
    QListView, QGraphicsView, QGraphicsScene, QSizePolicy,
    QGraphicsGridLayout, QGraphicsLayoutItem, QGraphicsSimpleTextItem,
    QGraphicsWidget, QGraphicsLineItem
)
from AnyQt.QtGui import QPainter, QColor, QPen
from AnyQt.QtCore import Qt, QEvent, QSizeF, QItemSelectionModel

import Orange
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils import itemmodels

from ..utils import group as exp

from .utils.settings import SetContextHandler


def take_columns(data, indices):
    """
    Return an array from the data's columns indexed by indices.

    None values in `indices` are interpreted as indexing a missing
    `column`, i.e. all the values are NaN.

    The resulting array is of shape (N * k,) where the data from
    column 0 is located in `res[:N]`, column2 in `res[N: 2 * N]`, ...
    """
    columns = []
    for index in indices:
        if index is not None:
            col_data, _ = data.get_column_view(index)
        else:
            col_data = numpy.full(len(data), numpy.nan)
        columns.append(col_data)
    return numpy.hstack(columns)


def dist_eucl(a1, a2):
    mask = ~(numpy.isnan(a1) | numpy.isnan(a2))
    a1, a2 = a1[mask], a2[mask]
    return numpy.sqrt(numpy.sum((a1 - a2) ** 2))


def dist_pcorr(a1, a2):
    mask = ~(numpy.isnan(a1) | numpy.isnan(a2))
    a1, a2 = a1[mask], a2[mask]
    P = numpy.corrcoef(a1, a2)[0, 1]
    return (1.0 - P) / 2.0


def dist_spearman(a1, a2):
    mask = ~(numpy.isnan(a1) | numpy.isnan(a2))
    a1, a2 = a1[mask], a2[mask]
    R, _ = scipy.stats.spearmanr(a1, a2)
    return (1.0 - R) / 2.0


@contextmanager
def widget_disable(widget):
    """A context to disable the widget (enabled property)
    """
    widget.setEnabled(False)
    try:
        yield
    finally:
        widget.setEnabled(True)


@contextmanager
def disable_updates(widget):
    """
    A context that sets '_disable_updates' member to True, and
    then restores it.

    """
    widget._disable_updates = True
    try:
        yield
    finally:
        widget._disable_updates = False


def group_label(splits, groups):
    """Return group label.
    """
    labels = ["%s=%s" % (split, group) \
              for split, group in zip(splits, groups)]
    return " | ".join(labels)


def sort_label(sort, attr):
    """Return within group sorted items label for attribute.
    """
    items = [(key, attr.attributes.get(key, "?")) \
             for key in sort]
    labels = ["%s=%s" % tuple(item) for item in items]
    return " | ".join(labels)


def float_if_posible(val):
    """Return val as float if possible otherwise return the value unchanged.
    """
    try:
        return float(val)
    except ValueError:
        return val


def experiment_description(feature):
    """Return experiment description from ``feature.attributes``.
    """
    text = ""
    if feature.attributes:
        items = feature.attributes.items()
        items = [(escape(key), escape(value)) for key, value in items]
        labels = map("%s = %s".__mod__, items)
        text += "<b>%s</b><br/>" % escape(feature.name)
        text += "<br/>".join(labels)
    return text


class OWQualityControl(widget.OWWidget):
    name = "Quality Control"
    description = "Experiment quality control"
    icon = "../widgets/icons/QualityControl.svg"
    priority = 5000

    inputs = [("Experiment Data", Orange.data.Table, "set_data")]
    outputs = []

    DISTANCE_FUNCTIONS = [("Distance from Pearson correlation",
                           dist_pcorr),
                          ("Euclidean distance",
                           dist_eucl),
                          ("Distance from Spearman correlation",
                           dist_spearman)]

    settingsHandler = SetContextHandler()

    split_by_labels = settings.ContextSetting({})
    sort_by_labels = settings.ContextSetting({})

    selected_distance_index = settings.Setting(0)

    def __init__(self, parent=None):
        super().__init__(parent)

        ## Attributes
        self.data = None
        self.distances = None
        self.groups = None
        self.unique_pos = None
        self.base_group_index = 0

        ## GUI
        box = gui.widgetBox(self.controlArea, "Info")
        self.info_box = gui.widgetLabel(box, "\n")

        ## Separate By box
        box = gui.widgetBox(self.controlArea, "Separate By")
        self.split_by_model = itemmodels.PyListModel(parent=self)
        self.split_by_view = QListView()
        self.split_by_view.setSelectionMode(QListView.ExtendedSelection)
        self.split_by_view.setModel(self.split_by_model)
        box.layout().addWidget(self.split_by_view)

        self.split_by_view.selectionModel().selectionChanged.connect(
            self.on_split_key_changed)

        ## Sort By box
        box = gui.widgetBox(self.controlArea, "Sort By")
        self.sort_by_model = itemmodels.PyListModel(parent=self)
        self.sort_by_view = QListView()
        self.sort_by_view.setSelectionMode(QListView.ExtendedSelection)
        self.sort_by_view.setModel(self.sort_by_model)
        box.layout().addWidget(self.sort_by_view)

        self.sort_by_view.selectionModel().selectionChanged.connect(
            self.on_sort_key_changed)

        ## Distance box
        box = gui.widgetBox(self.controlArea, "Distance Measure")
        gui.comboBox(box, self, "selected_distance_index",
                     items=[name for name, _ in self.DISTANCE_FUNCTIONS],
                     callback=self.on_distance_measure_changed)

        self.scene = QGraphicsScene()
        self.scene_view = QGraphicsView(self.scene)
        self.scene_view.setRenderHints(QPainter.Antialiasing)
        self.scene_view.setAlignment(Qt.AlignLeft | Qt.AlignVCenter)
        self.mainArea.layout().addWidget(self.scene_view)

        self.scene_view.installEventFilter(self)

        self._disable_updates = False
        self._cached_distances = {}
        self._base_index_hints = {}
        self.main_widget = None

        self.resize(800, 600)

    def clear(self):
        """Clear the widget state."""
        self.data = None
        self.distances = None
        self.groups = None
        self.unique_pos = None

        with disable_updates(self):
            self.split_by_model[:] = []
            self.sort_by_model[:] = []

        self.main_widget = None
        self.scene.clear()
        self.info_box.setText("\n")
        self._cached_distances = {}

    def set_data(self, data=None):
        """Set input experiment data."""
        self.closeContext()
        self.clear()

        self.error(0)
        self.warning(0)

        if data is not None:
            keys = self.get_suitable_keys(data)
            if not keys:
                self.error(0, "Data has no suitable feature labels.")
                data = None

        self.data = data
        if data is not None:
            self.on_new_data()

    def update_label_candidates(self):
        """Update the label candidates selection GUI 
        (Group/Sort By views).

        """
        keys = self.get_suitable_keys(self.data)
        with disable_updates(self):
            self.split_by_model[:] = keys
            self.sort_by_model[:] = keys

    def get_suitable_keys(self, data):
        """ Return suitable attr label keys from the data where
        the key has at least two unique values in the data.

        """
        attrs = [attr.attributes.items() for attr in data.domain.attributes]
        attrs = reduce(operator.iadd, attrs, [])
        # in case someone put non string values in attributes dict
        attrs = [(str(key), str(value)) for key, value in attrs]
        attrs = set(attrs)
        values = defaultdict(set)
        for key, value in attrs:
            values[key].add(value)
        keys = [key for key in values if len(values[key]) > 1]
        return keys

    def selected_split_by_labels(self):
        """Return the current selected split labels.
        """
        sel_m = self.split_by_view.selectionModel()
        indices = [r.row() for r in sel_m.selectedRows()]
        return [self.sort_by_model[i] for i in indices]

    def selected_sort_by_labels(self):
        """Return the current selected sort labels
        """
        sel_m = self.sort_by_view.selectionModel()
        indices = [r.row() for r in sel_m.selectedRows()]
        return [self.sort_by_model[i] for i in indices]

    def selected_distance(self):
        """Return the selected distance function.
        """
        return self.DISTANCE_FUNCTIONS[self.selected_distance_index][1]

    def selected_base_group_index(self):
        """Return the selected base group index
        """
        return self.base_group_index

    def selected_base_indices(self, base_group_index=None):
        indices = []
        for g, ind in self.groups:
            if base_group_index is None:
                label = group_label(self.selected_split_by_labels(), g)
                ind = [i for i in ind if i is not None]
                i = self._base_index_hints.get(label, ind[0] if ind else None)
            else:
                i = ind[base_group_index]
            indices.append(i)
        return indices

    def on_new_data(self):
        """We have new data and need to recompute all.
        """
        self.closeContext()

        self.update_label_candidates()
        self.info_box.setText(
            "%s genes \n%s experiments" %
            (len(self.data),  len(self.data.domain.attributes))
        )

        self.base_group_index = 0

        keys = self.get_suitable_keys(self.data)
        self.openContext(keys)

        ## Restore saved context settings (split/sort selection)
        split_by_labels = self.split_by_labels
        sort_by_labels = self.sort_by_labels

        def select(model, selection_model, selected_items):
            """Select items in a Qt item model view
            """
            all_items = list(model)
            try:
                indices = [all_items.index(item) for item in selected_items]
            except:
                indices = []
            for ind in indices:
                selection_model.select(model.index(ind),
                                       QItemSelectionModel.Select)

        with disable_updates(self):
            select(self.split_by_view.model(),
                   self.split_by_view.selectionModel(),
                   split_by_labels)

            select(self.sort_by_view.model(),
                   self.sort_by_view.selectionModel(),
                   sort_by_labels)

        with widget_disable(self):
            self.split_and_update()

    def on_split_key_changed(self, *args):
        """Split key has changed
        """
        with widget_disable(self):
            if not self._disable_updates:
                self.base_group_index = 0
                self.split_by_labels = self.selected_split_by_labels()
                self.split_and_update()

    def on_sort_key_changed(self, *args):
        """Sort key has changed
        """
        with widget_disable(self):
            if not self._disable_updates:
                self.base_group_index = 0
                self.sort_by_labels = self.selected_sort_by_labels()
                self.split_and_update()

    def on_distance_measure_changed(self):
        """Distance measure has changed
        """
        if self.data is not None:
            with widget_disable(self):
                self.update_distances()
                self.replot_experiments()

    def on_view_resize(self, size):
        """The view with the quality plot has changed
        """
        if self.main_widget:
            current = self.main_widget.size()
            self.main_widget.resize(size.width() - 6,
                                    current.height())

            self.scene.setSceneRect(self.scene.itemsBoundingRect())

    def on_rug_item_clicked(self, item):
        """An ``item`` in the quality plot has been clicked.
        """
        update = False
        sort_by_labels = self.selected_sort_by_labels()
        if sort_by_labels and item.in_group:
            ## The item is part of the group
            if item.group_index != self.base_group_index:
                self.base_group_index = item.group_index
                update = True

        else:
            if sort_by_labels:
                # If the user clicked on an background item it
                # invalidates the sorted labels selection
                with disable_updates(self):
                    self.sort_by_view.selectionModel().clear()
                    update = True

            index = item.index
            group = item.group
            label = group_label(self.selected_split_by_labels(), group)

            if self._base_index_hints.get(label, 0) != index:
                self._base_index_hints[label] = index
                update = True

        if update:
            with widget_disable(self):
                self.split_and_update()

    def eventFilter(self, obj, event):
        if obj is self.scene_view and event.type() == QEvent.Resize:
            self.on_view_resize(event.size())
        return super().eventFilter(obj, event)

    def split_and_update(self):
        """
        Split the data based on the selected sort/split labels
        and update the quality plot.

        """
        split_labels = self.selected_split_by_labels()
        sort_labels = self.selected_sort_by_labels()

        self.warning(0)
        if not split_labels:
            self.warning(0, "No separate by label selected.")

        self.groups, self.unique_pos = \
                exp.separate_by(self.data, split_labels,
                                consider=sort_labels,
                                add_empty=True)

        self.groups = sorted(self.groups.items(),
                             key=lambda t: list(map(float_if_posible, t[0])))
        self.unique_pos = sorted(self.unique_pos.items(),
                                 key=lambda t: list(map(float_if_posible, t[0])))

        if self.groups:
            if sort_labels:
                group_base = self.selected_base_group_index()
                base_indices = self.selected_base_indices(group_base)
            else:
                base_indices = self.selected_base_indices()
            self.update_distances(base_indices)
            self.replot_experiments()

    def get_cached_distances(self, measure):
        if measure not in self._cached_distances:
            attrs = self.data.domain.attributes
            mat = numpy.zeros((len(attrs), len(attrs)))

            self._cached_distances[measure] = \
                (mat, set(zip(range(len(attrs)), range(len(attrs)))))

        return self._cached_distances[measure]

    def get_cached_distance(self, measure, i, j):
        matrix, computed = self.get_cached_distances(measure)
        key = (i, j) if i < j else (j, i)
        if key in computed:
            return matrix[i, j]
        else:
            return None

    def get_distance(self, measure, i, j):
        d = self.get_cached_distance(measure, i, j)
        if d is None:
            vec_i = take_columns(self.data, [i])
            vec_j = take_columns(self.data, [j])
            d = measure(vec_i, vec_j)

            mat, computed = self.get_cached_distances(measure)
            mat[i, j] = d
            key = key = (i, j) if i < j else (j, i)
            computed.add(key)
        return d

    def store_distance(self, measure, i, j, dist):
        matrix, computed = self.get_cached_distances(measure)
        key = (i, j) if i < j else (j, i)
        matrix[j, i] = matrix[i, j] = dist
        computed.add(key)

    def update_distances(self, base_indices=()):
        """Recompute the experiment distances.
        """
        distance = self.selected_distance()
        if base_indices == ():
            base_group_index = self.selected_base_group_index()
            base_indices = [ind[base_group_index] \
                            for _, ind in self.groups]

        assert(len(base_indices) == len(self.groups))

        base_distances = []
        attributes = self.data.domain.attributes
        pb = gui.ProgressBar(self, len(self.groups) * len(attributes))

        for (group, indices), base_index in zip(self.groups, base_indices):
            # Base column of the group
            if base_index is not None:
                base_vec = take_columns(self.data, [base_index])
                distances = []
                # Compute the distances between base column
                # and all the rest data columns.
                for i in range(len(attributes)):
                    if i == base_index:
                        distances.append(0.0)
                    elif self.get_cached_distance(distance, i, base_index) is not None:
                        distances.append(self.get_cached_distance(distance, i, base_index))
                    else:
                        vec_i = take_columns(self.data, [i])
                        dist = distance(base_vec, vec_i)
                        self.store_distance(distance, i, base_index, dist)
                        distances.append(dist)
                    pb.advance()

                base_distances.append(distances)
            else:
                base_distances.append(None)

        pb.finish()
        self.distances = base_distances

    def replot_experiments(self):
        """Replot the whole quality plot.
        """
        self.scene.clear()
        labels = []

        max_dist = numpy.nanmax(list(filter(None, self.distances)))
        rug_widgets = []

        group_pen = QPen(Qt.black)
        group_pen.setWidth(2)
        group_pen.setCapStyle(Qt.RoundCap)
        background_pen = QPen(QColor(0, 0, 250, 150))
        background_pen.setWidth(1)
        background_pen.setCapStyle(Qt.RoundCap)

        main_widget = QGraphicsWidget()
        layout = QGraphicsGridLayout()
        attributes = self.data.domain.attributes
        if self.data is not None:
            for (group, indices), dist_vec in zip(self.groups, self.distances):
                indices_set = set(indices)
                rug_items = []
                if dist_vec is not None:
                    for i, attr in enumerate(attributes):
                        # Is this a within group distance or background
                        in_group = i in indices_set
                        if in_group:
                            rug_item = ClickableRugItem(dist_vec[i] / max_dist,
                                           1.0, self.on_rug_item_clicked)
                            rug_item.setPen(group_pen)
                            tooltip = experiment_description(attr)
                            rug_item.setToolTip(tooltip)
                            rug_item.group_index = indices.index(i)
                            rug_item.setZValue(rug_item.zValue() + 1)
                        else:
                            rug_item = ClickableRugItem(dist_vec[i] / max_dist,
                                           0.85, self.on_rug_item_clicked)
                            rug_item.setPen(background_pen)
                            tooltip = experiment_description(attr)
                            rug_item.setToolTip(tooltip)

                        rug_item.group = group
                        rug_item.index = i
                        rug_item.in_group = in_group

                        rug_items.append(rug_item)

                rug_widget = RugGraphicsWidget(parent=main_widget)
                rug_widget.set_rug(rug_items)

                rug_widgets.append(rug_widget)

                label = group_label(self.selected_split_by_labels(), group)
                label_item = QGraphicsSimpleTextItem(label, main_widget)
                label_item = GraphicsSimpleTextLayoutItem(label_item, parent=layout)
                label_item.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
                labels.append(label_item)

        for i, (label, rug_w) in enumerate(zip(labels, rug_widgets)):
            layout.addItem(label, i, 0, Qt.AlignVCenter)
            layout.addItem(rug_w, i, 1)
            layout.setRowMaximumHeight(i, 30)

        main_widget.setLayout(layout)
        self.scene.addItem(main_widget)
        self.main_widget = main_widget
        self.rug_widgets = rug_widgets
        self.labels = labels
        self.on_view_resize(self.scene_view.size())


class GraphicsSimpleTextLayoutItem(QGraphicsLayoutItem):
    """ A Graphics layout item wrapping a QGraphicsSimpleTextItem alowing it
    to be managed by a layout.

    """
    def __init__(self, text_item, orientation=Qt.Horizontal, parent=None):
        super().__init__(parent)
        self.orientation = orientation
        self.text_item = text_item
        if orientation == Qt.Vertical:
            self.text_item.rotate(-90)
            self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        else:
            self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)

    def setGeometry(self, rect):
        super().setGeometry(rect)
        if self.orientation == Qt.Horizontal:
            self.text_item.setPos(rect.topLeft())
        else:
            self.text_item.setPos(rect.bottomLeft())

    def sizeHint(self, which, constraint=QSizeF()):
        if which in [Qt.PreferredSize]:
            size = self.text_item.boundingRect().size()
            if self.orientation == Qt.Horizontal:
                return size
            else:
                return QSizeF(size.height(), size.width())
        else:
            return QSizeF()

    def updateGeometry(self):
        super().updateGeometry()
        parent = self.parentLayoutItem()
        if parent.isLayout():
            parent.updateGeometry()

    def setFont(self, font):
        self.text_item.setFont(font)
        self.updateGeometry()

    def setText(self, text):
        self.text_item.setText(text)
        self.updateGeometry()


class RugGraphicsWidget(QGraphicsWidget):
    def __init__(self, parent=None, rug=None):
        QGraphicsWidget.__init__(self, parent)
        self.rug_items = []
        self.set_rug(rug)
        self.setMaximumHeight(30)
        self.setMinimumHeight(30)

    def clear(self):
        """
        Clear all rug items from this widget and remove them 
        from the scene.

        """
        for item in self.rug_items:
            item.setParent(None)
            if self.scene() is not None:
                self.scene().removeItem(item)

    def set_rug(self, rug):
        """
        Set the rug items.

        ``rug`` must be a list of floats or already initialized
        instances of RugItem. The widget takes ownership of all
        items.

        """
        rug = rug if rug is not None else []
        self.clear()
        self.add_rug(rug)

    def add_rug(self, rug):
        """
        Add rug items.

        See :obj:`set_rug`

        """
        items = []
        for item in rug:
            if isinstance(item, float):
                item = RugItem(value=item)
                items.append(item)
            elif isinstance(item, RugItem):
                items.append(item)

        for item in items:
            item.setParentItem(self)

        self.rug_items.extend(items)

        self.update_rug_geometry()

    def update_rug_geometry(self):
        """Recompute the rug items positions within this widget.
        """
        size = self.size()
        height = size.height()
        width = size.width()

        for item in self.rug_items:
            offset = (1.0 - item.height) * height / 2.0
            item.setPos(width * item.value, 0)
            item.setLine(0., offset, 0., height - offset)

    def resizeEvent(self, event):
        """Reimplemented from QGraphicsWidget
        """
        QGraphicsWidget.resizeEvent(self, event)
        self.update_rug_geometry()


class RugItem(QGraphicsLineItem):
    def __init__(self, value, height):
        QGraphicsLineItem.__init__(self)
        self.value = value
        self.height = height

    def set_height(self, height):
        """Set the height of this item (in ratio of the rug height)
        """
        self.height = height


class ClickableRugItem(RugItem):
    def __init__(self, value, height, on_pressed):
        RugItem.__init__(self, value, height)
        self.on_pressed = on_pressed
        self.setAcceptedMouseButtons(Qt.LeftButton)
        self.setAcceptHoverEvents(True)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton and self.on_pressed:
            self.on_pressed(self)

    def hoverEnterEvent(self, event):
        pen = QPen(self.pen())
        pen.setWidthF(3)
        self.setPen(pen)
        return RugItem.hoverEnterEvent(self, event)

    def hoverLeaveEvent(self, event):
        pen = QPen(self.pen())
        pen.setWidth(2)
        self.setPen(pen)
        return RugItem.hoverLeaveEvent(self, event)


def test_main(argv=sys.argv):
    from AnyQt.QtWidgets import QApplication
    if len(argv) > 1:
        filename = argv[1]
    else:
        filename = "GDS624.tab"
    data = Orange.data.Table(filename)

    app = QApplication(argv)
    w = OWQualityControl()
    w.set_data(data)
    w.handleNewSignals()
    w.show()
    w.raise_()
    r = app.exec_()
    w.set_data(None)
    w.handleNewSignals()
    w.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())



import sys
import warnings
import io
from collections import defaultdict
from functools import partial
import itertools
import xml.sax

import six

from AnyQt.QtWidgets import (
    QWidget, QComboBox, QListView, QLineEdit, QPlainTextEdit, QRadioButton,
    QButtonGroup, QGroupBox, QCheckBox, QLabel, QFrame, QTabWidget,
    QSizePolicy, QGridLayout, QVBoxLayout, QHBoxLayout, QStackedLayout,
)
from AnyQt.QtGui import QRegExpValidator

from AnyQt.QtCore import (
    Qt, QObject, QRegExp, QModelIndex, QThread, QThreadPool, QSize,
    QStringListModel, QItemSelectionModel
)
from AnyQt.QtCore import Slot

import Orange

from Orange.widgets.utils import concurrent
from Orange.widgets import widget, gui, settings

from .. import biomart


def is_hidden(tree):
    return (getattr(tree, "hidden", "false") != "false" or
            getattr(tree, "hideDisplay", "false") != "false")


class Control(object):

    """ Base mixin class for query GUI widgets
    """

    def __init__(self, tree=None, dataset=None, master=None, **kwargs):
        """
            :param tree: configuration element
            :type tree: biomart.ConfigurationNode
            :param dataset: dataset
            :type dataset: biomart.BioMartDataset
            :param master: main widget
            :type master: OWBioMart

        """
        super(Control, self).__init__(**kwargs)
        self.tree = tree
        self.dataset = dataset
        self.master = master
        self.subControls = []

        if tree is not None and isinstance(self, QObject):
            self.setObjectName(tree.internalName)
            if hasattr(tree, "description"):
                self.setToolTip(tree.description)

    def addSubControl(self, tree, control):
        self.subControls.append((tree, control))

    def registerDelayedCall(self, call):
        self.master.registerDelayedCall(call)

    def pushAction(self, action):
        self.master.pushAction(action)

    def query(self):
        return itertools.chain(*[ctrl.query() for _, ctrl in self.subControls])

    def setControlValue(self, name, value):
        if "." in name:
            name, rest = name.split(".", 1)
            controls = {tree.internalName: control
                        for tree, control in self.subControls}
            ctrl = controls.get("name", None)
            if ctrl:
                ctrl.setControlValue(rest, value)


class UnknownFilter(QWidget, Control):

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)


class TextFieldFilter(QLineEdit, Control):

    """ A single edit line filter
    """

    def __init__(self, tree, dataset, master, parent=None):
        QLineEdit.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        if hasattr(tree, "regexp"):
            self.setValidator(QRegExpValidator(QRegExp(tree.regexp), self))

        if hasattr(tree, "defaultValue"):
            self.setText(tree.defaultValue)

    def get_filter(self):
        return self.tree.internalName, str(self.text())

    def query(self):
        return [("Filter", self.tree, str(self.text()))]

    def setControlValue(self, name, value):
        self.setText(value)


class IdListFilter(QWidget, Control):

    """ Multiple ids filter
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.tree = tree
        self.dataset = dataset
        self.master = master
        self.setObjectName(tree.internalName)

        self.setLayout(QGridLayout())
        self.textWidget = QPlainTextEdit()  # TODO: Subclass to receive drop events from item model views
        self.layout().addWidget(self.textWidget, 0, 0, 1, 1)

    def value(self):
        """ Return filter value for use in a query
        """
        return str(self.textWidget.toPlainText()).split()

    def get_filter(self):
        return self.tree.internalName, self.value()

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        if type(value) == list:
            value = "\n".join(value)
        self.textWidget.setPlainText(value)


class RadioBooleanFilter(QWidget, Control):

    """ Boolean filter (Only/Exclude)
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setLayout(QVBoxLayout())
        self.buttonGroup = QButtonGroup(self)
        self.values = []
        for i, option in enumerate(tree.subelements_top("Option")):
            rb = QRadioButton(option.displayName, self)
            self.buttonGroup.addButton(rb)
            self.buttonGroup.setId(rb, i)
            self.layout().addWidget(rb)
            self.values.append(option.value)
        self.buttonGroup.button(0).setChecked(True)

    def value(self):
        return {"excluded": "%i" % self.buttonGroup.checkedId()}

    def get_filter(self):
        return self.tree.internalName, self.value()

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        for i, v in enumerate(self.values):
            if v == value:
                button = self.buttonGroup.button(i)
                button.setChecked(True)
                break


class DropDownFilter(QComboBox, Control):

    """ List menu filter
    """

    def __init__(self, tree, dataset, master, parent=None):
        QComboBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.options = []
        self.selectedIndex = 0

        if getattr(tree, "graph", "0") == "1":
            self.setOptions(tree.subelements("Option"))
        else:
            self.setOptions(tree.subelements_top("Option"))

        self.currentIndexChanged[int].connect(self.onIndexChange)

    def setOptions(self, options):
        self.options = []
        self.blockSignals(True)
        self.clear()
        for option in options:
            self.addItem(option.displayName)
            self.options.append(option)
        self.selectedIndex = 0
        self.blockSignals(False)
        self.registerDelayedCall(lambda: self.onIndexChange(0))

    def onIndexChange(self, index):
        if self.options:
            option = self.options[index]
            self.selectedIndex = index
            pushActions = option.subelements_top("PushAction")
            for action in pushActions:
                self.master.pushAction(action)

    def value(self):
        option = self.options[self.selectedIndex]
        return option.value

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        for i, option in enumerate(self.options):
            if option.value == value:
                self.setCurrentIndex(i)


def rows(index_list):
    return map(QModelIndex.row, index_list)


class MultiSelectListFilter(QListView, Control):

    """ Menu list filter with multiple selection
    """

    def __init__(self, tree, dataset, master, parent=None):
        QComboBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setSelectionMode(QListView.ExtendedSelection)
        model = QStringListModel(self)
        self.setModel(model)
        self.setOptions(tree.subelements_top("Option"))

    def setOptions(self, options):
        self.options = []
        for option in options:
            self.options.append(option)
        self.model().setStringList(
            [option.displayName for option in self.options])

    def value(self):
        value = []
        for index in rows(self.selectedIndexes()):
            value.append(self.options[index].value)
        return ",".join(value)

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        if isinstance(value, str):
            values = value.split(",")
        else:
            values = value
        selection = self.selectionModel()
        for i, option in enumerate(self.options):
            if option.value in values:
                selection.select(self.model().index(i),
                                 QItemSelectionModel.Select)


class DropDownRadioBooleanFilter(QWidget, Control):
    """Container for multiple boolean filters
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setLayout(QHBoxLayout())
        self.cb = QComboBox(self)

        self.layout().addWidget(self.cb)

        rblayout = QVBoxLayout()
        self.radioButtons = [QRadioButton("Only", self),
                             QRadioButton("Excluded", self)
                             ]

        for b in self.radioButtons:
            rblayout.addWidget(b)

        self.radioButtons[0].setChecked(True)

        self.layout().addLayout(rblayout)

        self.options = []

        self.setOptions(tree.subelements_top("Option"))

    def setOptions(self, options):
        self.cb.clear()
        self.options = []
        for option in options:
            self.cb.addItem(option.displayName)
            self.options.append(option)

        for op, rb in zip(self.options[0].subelements_top("Option"),
                          self.radioButtons):
            rb.setText(op.displayName)
            rb.setChecked(getattr(op, "default", "false") == "true")

    def value(self):
        return {"excluded": "0" if self.radioButtons[0].isChecked() else "1"}

    def query(self):
        filter = self.options[self.cb.currentIndex()]
        filter = biomart.FilterDescription(
            self.tree.registry, "FilterDescription",
            filter.attributes, filter.children)
        return [("Filter", filter, self.value())]

    def setControlValue(self, name, value):
        for i, option in enumerate(self.options):
            if option.internalName == name:
                self.cb.setCurrentIndex(i)
                if value == "Only":
                    self.radioButtons[0].setChecked(True)


class DropDownIdListFilter(QWidget, Control):

    """Container for multiple id list filters
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setLayout(QVBoxLayout())
        self.setContentsMargins(0, 0, 0, 0)
        self.cb = QComboBox()
        self.idsEdit = QPlainTextEdit()

        self.layout().addWidget(self.cb)
        self.layout().addWidget(self.idsEdit)

        self.options = []
        self.setOptions(tree.subelements_top("Option"))

    def setOptions(self, options):
        self.cb.clear()
        self.options = []
        for option in options:
            self.cb.addItem(option.displayName)
            self.options.append(option)

    def value(self):
        return str(self.idsEdit.toPlainText()).split()

    def query(self):
        filter = self.options[self.cb.currentIndex()]
        filter = biomart.FilterDescription(
            self.tree.registry, "FilterDescription",
            filter.attributes, filter.children)
        return [("Filter", filter, self.value())]

    def setControlValue(self, name, value):
        if isinstance(value, list):
            value = "\n".join(value)

        for i, op in enumerate(self.options):
            if name == op.internalName:
                self.cb.setCurrentIndex(i)
                self.idsEdit.setPlainText(value)


class CollectionWidget(QGroupBox, Control):
    NO_FLAGS = 0
    FILTER_FLAG = 1
    ATTRIBUTE_FLAG = 2
    SINGLE_FILTER_FLAG = 4

    def __init__(self, tree, dataset, master, parent=None):
        QGroupBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setFlat(True)
        self.attributes = []
        self.filters = []

    def setCollection(self, tree, dataset, flags=0):
        self.tree = tree
        self.dataset = dataset
        self.collectionFlags = flags


def isFilterCollection(tree):
    """ In case all AttributeDescription nodes contain pointers to filters
    (case of downstream/upstream_flank)
    """
    if tree.tag == "FilterCollection":
        return True
    return all(["pointerFilter" in desc.attributes
                for desc in tree.elements("AttributeDescription")])


class AttributeWidget(QCheckBox, Control):

    """ Single attribute selection widget
    """

    def __init__(self, tree, dataset, master, parent=None):
        QCheckBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setText(getattr(tree, "displayName", ""))
        self.setChecked(getattr(tree, "default", "false") == "true")

        if hasattr(tree, "description"):
            self.setToolTip(tree.description)

    def query(self):
        if self.isChecked():
            return [("Attribute", self.tree, self.isChecked())]
        else:
            return []

    def setControlValue(self, name, value):
        self.setChecked(value == "true")


class AttributeCollectionWidget(CollectionWidget):

    def __init__(self, tree, dataset, master, parent=None):
        CollectionWidget.__init__(self, tree, dataset, master, parent)
        self.setLayout(QGridLayout(self))
        self.setCollection(tree, dataset)

    def setCollection(self, collection, dataset, flags=0):
        CollectionWidget.setCollection(self, collection, dataset, flags)
        self.setTitle(getattr(collection, "displayName", ""))

        if hasattr(collection, "description"):
            self.setToolTip(collection.description)

        attributes = [attr for attr in collection.elements("AttributeDescription")
                      if not is_hidden(attr)]

        numAttrs = max(len(attributes) / 2 + 1, 1)
        i = 0
        for attr in attributes:
            if attr.is_pointer():
                try:
                    dataset, newattr = attr.get_pointed()
                except ValueError as ex:
                    newattr = None

                if not newattr:
                    continue
                attr = newattr

            attr_widget = AttributeWidget(attr, self.dataset, self.master, self)
            self.layout().addWidget(attr_widget, i % numAttrs, i / numAttrs)
            self.addSubControl(attr, attr_widget)
            i += 1


def filterType(filter):
    return (getattr(filter, "displayType", ""),
            getattr(filter, "style", ""),
            getattr(filter, "multipleValues", ""))


class FilterCollectionWidget(CollectionWidget):

    def __init__(self, tree, dataset, master, parent=None):
        CollectionWidget.__init__(self, tree, dataset, master, parent)
        self.setLayout(QGridLayout())

        self.layout().setContentsMargins(5, 5, 5, 5)
        self.layout().setColumnStretch(0, 10)
        self.layout().setColumnStretch(1, 10)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        self.setCollection(tree, dataset)

    def setCollection(self, collection, dataset, flags=0):
        CollectionWidget.setCollection(self, collection, dataset, flags)
        filters = [f for f in collection.elements("FilterDescription") if not is_hidden(f)] + \
                  [f for f in collection.elements("AttributeDescription") if not is_hidden(f)]  # in case of pointers to filters (upstream/downstream flank)
        self.enableCB = QCheckBox(getattr(collection, "displayName", ""), self)

        if hasattr(collection, "description"):
            self.setToolTip(collection.description)

        if len(filters) == 1 and (not hasattr(filters[0], "displayName") or \
                                  getattr(filters[0], "displayName", "") == getattr(collection, "displayName", "")):
            flags = flags | self.SINGLE_FILTER_FLAG

        i = 0
        if not flags & self.SINGLE_FILTER_FLAG:
            self.layout().addWidget(self.enableCB, 0, 0)
            i += 1

        for filter in filters:
            fType = getattr(filter, "type", None)
            if filter.is_pointer():
                try:
                    dataset, newfilter = filter.get_pointed()
                except ValueError as ex:
                    newfilter = None

                if not newfilter:
                    continue
                filter = newfilter
                fType = getattr(filter, "type", None) if fType is None else fType
            label, filter_widget = self.buildFilter(filter, flags, fTypeHint=fType)
            if isinstance(label, six.string_types):
                label = QLabel(label)

            self.layout().addWidget(label, i, 0)
            self.layout().addWidget(filter_widget, i, 1)
            i += 1

            self.addSubControl(filter, filter_widget)
        if self.layout().count() == 0:
            self.layout().addWidget(self.enableCB, 0, 0)

    def buildFilter(self, filter, flags, fTypeHint=None):
        if flags & self.SINGLE_FILTER_FLAG:
            label = self.enableCB
        else:
            label = getattr(filter, "displayName", "")

        fType = filterType(filter)
        if fTypeHint == "drop_down_basic_filter":
            fType = ("list", "menu", "")
        filter_widget = None

        if fType == ("text", "", ""):
            filter_widget = TextFieldFilter(filter, self.dataset, self.master, self)

        elif fType == ("text", "", "1") or fType == ("text", "", "true"):
            filter_widget = IdListFilter(filter, self.dataset, self.master, self)

        elif fType == ("list", "radio", ""):
            filter_widget = RadioBooleanFilter(filter, self.dataset, self.master, self)

        elif fType == ("list", "menu", ""):
            filter_widget = DropDownFilter(filter, self.dataset, self.master, self)

        elif fType == ("list", "menu", "1") or fType == ("list", "menu", "true"):
            filter_widget = MultiSelectListFilter(filter, self.dataset, self.master, self)

        elif fType == ("container", "", ""):
            fType = set(map(filterType, filter.elements_top("Option")))
            if len(fType) != 1:
                warnings.warn("Multiple filter types in a single container!" + str(fType))
            fType = fType.pop()
            if fType[0] == "text":  # ("text", "", "1"):
                filter_widget = DropDownIdListFilter(filter, self.dataset, self.master, self)
            elif fType == ("list", "radio", ""):
                filter_widget = DropDownRadioBooleanFilter(filter, self.dataset, self.master, self)

        if filter_widget is None:
            warnings.warn("Unknown filter type '%s' %s'" % (repr(fType), repr(filter)))
            filter_widget = UnknownFilter(filter, self.dataset, self.master, self)

        filter_widget.setMaximumWidth(400)
        return label, filter_widget

    def query(self):
        if self.enableCB.isChecked():
            return CollectionWidget.query(self)
        else:
            return []


class GroupWidget(QGroupBox, Control):

    def __init__(self, tree, dataset, master, parent=None):
        QGroupBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setLayout(QVBoxLayout())

        if tree:
            self.setGroup(tree)

    def setGroup(self, group):
        self.setTitle(getattr(group, "displayName", ""))
        if hasattr(group, "description"):
            self.setToolTip(group.description)
        self.setCheckable(True)
        self.setChecked(True)

        if group.tag == "FilterGroup":
            collections = group.elements("FilterCollection")
        else:
            collections = group.elements("AttributeCollection")
        for collection in collections:
            if isFilterCollection(collection):
                collection_widget = FilterCollectionWidget(collection, self.dataset, self.master, self)
            else:
                collection_widget = AttributeCollectionWidget(collection, self.dataset, self.master, self)

            self.layout().addWidget(collection_widget)
            self.addSubControl(collection, collection_widget)


class PageWidget(QFrame, Control):

    def __init__(self, tree, dataset, master, parent=None):
        QFrame.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setLayout(QVBoxLayout())
        self.outFormats = getattr(tree, "outFormats", "tsv")

        if tree:
            self.setPage(tree)

    def setPage(self, tree):
        if tree.tag == "FilterPage":
            groups = tree.elements("FilterGroup")
        else:
            groups = tree.elements("AttributeGroup")

        for group in groups:
            group_widget = GroupWidget(group, self.dataset, self.master, self)
            self.layout().addWidget(group_widget)
            self.addSubControl(group, group_widget)

        self.layout().addStretch(10)

        if hasattr(tree, "description"):
            self.setToolTip(tree.description)

# A list of preset mart services available for selection from the pull
# down menu

MartServices = [
    ("Ensembl", "http://www.ensembl.org/biomart/martservice"),
    ("Ensembl Fungi", "http://fungi.ensembl.org/biomart/martservice"),
    ("Ensembl Plants", "http://plants.ensembl.org/biomart/martservice"),
    ("Pancreatic Expression Database",
     "http://www.pancreasexpression.org/biomart/martservice"),
    ("VectorBase", "http://biomart.vectorbase.org/biomart/martservice"),
    ("Phytozome", "https://phytozome.jgi.doe.gov/biomart/martservice"),
]


class OWBioMart(widget.OWWidget):
    name = "BioMart"
    description = "Query BioMart service"
    icon = "../widgets/icons/BioMart.svg"
    priority = 2010

    outputs = [("Data", Orange.data.Table)]

    SHOW_FILTERS = True

    selectedService = settings.Setting(MartServices[0][1])
    selectedDataset = settings.Setting(0)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.selectedDatabase = 0
        self.uniqueRows = True

        gui.button(gui.widgetBox(self.controlArea, "Cache", addSpace=True),
                   self, "Clear cache",
                   tooltip="Clear saved query results",
                   callback=self.clearCache)
        self.serviceindex = 0
        self.serviceCombo = gui.comboBox(
            self.controlArea, self, "serviceindex", "Mart Service",
            callback=self._setServiceUrl
        )
        for name, url in MartServices:
            self.serviceCombo.addItem(name, userData=url)
        idx = self.serviceCombo.findData(self.selectedService, Qt.UserRole)
        self.serviceCombo.setCurrentIndex(idx)
        # self.selectedService = self.serviceCombo.itemData(self.serviceCombo.currentItem())

        self.martsCombo = gui.comboBox(
            self.controlArea, self, "selectedDatabase", "Database",
            callback=self.setSelectedMart,
            addSpace=True)
        self.martsCombo.setMaximumWidth(250)

        self.datasetsCombo = gui.comboBox(
            self.controlArea, self, "selectedDataset", "Dataset",
            callback=self.setSelectedDataset,
            addSpace=True)

        self.datasetsCombo.setMaximumWidth(250)

        gui.rubber(self.controlArea)

        box = gui.widgetBox(self.controlArea, "Results")
        gui.checkBox(
            box, self, "uniqueRows", "Unique results only",
            tooltip="Return unique results only.",)

        self.commitButton = gui.button(
            box, self, "Get Results", callback=self.commit,
            tooltip="Query the BioMart server and output the results",
            autoDefault=True)

        self.commitButton.setEnabled(False)

        self.mainWidget = gui.widgetBox(
            self.mainArea, orientation=QStackedLayout())

        self.mainTab = QTabWidget()

        self.mainWidget.layout().addWidget(self.mainTab)

        self.attributesConfigurationBox = gui.createTabPage(self.mainTab, "Attributes")

        if self.SHOW_FILTERS:  # ??
            self.filtersConfigurationBox = gui.createTabPage(self.mainTab, "Filters")

        self.error(0)
        self.setEnabled(False)
        self._task = None
        self._executor = concurrent.ThreadExecutor(
            threadPool=QThreadPool(maxThreadCount=2)
        )
        service = self.selectedService
        self._task = task = concurrent.Task(
            function=partial(self._get_registry, url=service))
        task.resultReady.connect(self.setBioMartRegistry)
        task.exceptionReady.connect(self._handleException)
        self._executor.submit(task)
        self._setServiceUrl()
        self._afterInitQueue = []

        try:
            from Bio import SeqIO
            self.hasBiopython = True
        except ImportError:
            self.warning(100, "Biopython package not found.\nTo retrieve FASTA sequence data from BioMart install Biopython.")
            self.hasBiopython = False

    def sizeHint(self):
        return QSize(800, 600)

    def _setServiceUrl(self):
        service = self.serviceCombo.itemData(self.serviceCombo.currentIndex())
        if service is not None:
            self.selectedService = service
            self._task = task = concurrent.Task(
                function=partial(self._get_registry, url=service))
            task.resultReady.connect(self.setBioMartRegistry)
            task.exceptionReady.connect(self._handleException)
            self._executor.submit(task)

    @staticmethod
    def _get_registry(url=None, precache=True):
        if url is None:
            url = MartServices[0][1]
        con = biomart.BioMartConnection(address=url, timeout=30)
        reg = biomart.BioMartRegistry(con)
        if precache:
            _ = reg.marts()
        return reg

    @Slot(Exception)
    def _handleException(self, exception):
        assert(QThread.currentThread() is self.thread())
        print("Task failed with:", exception, file=sys.stderr)
        import logging
        log = logging.getLogger(__name__)
        log.exception("Error:", exc_info=exception)
        self.error(0, str(exception))
        self.setEnabled(True)

    @Slot(object)
    def setBioMartRegistry(self, registry):
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        self.registry = registry
        self.marts = [mart for mart in self.registry.marts()
                      if getattr(mart, "visible", "0") != "0"]

        self.martsCombo.clear()
        for mart in self.marts:
            self.martsCombo.addItem(mart.displayName)

    def setSelectedMart(self):
        self.mart = self.marts[self.selectedDatabase]
        self.error(0)
        self.setEnabled(False)

        self._task = task = concurrent.Task(function=self.mart.datasets)
        task.resultReady.connect(self.setBioMartDatasets)
        task.exceptionReady.connect(self._handleException)
        self._executor.submit(task)

    @Slot(object)
    def setBioMartDatasets(self, datasets):
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        self.datasets = [data for data in datasets if
                         getattr(data, "visible", "0") != "0"]
        self.datasetsCombo.clear()
        self.datasetsCombo.addItems([data.displayName for data in self.datasets])

    def setSelectedDataset(self):
        self.dataset = self.datasets[self.selectedDataset]
        self.error(0)
        self.setEnabled(False)

        def get_configuration(dataset):
            connection = dataset.connection
            stream = connection.configuration(
                dataset=dataset.internalName,
                virtualSchema=dataset.serverVirtualSchema)
            response = stream.read()
            return response

        self._task = task = concurrent.Task(
            function=partial(get_configuration, self.dataset))

        task.resultReady.connect(self.setBioMartConfiguration)
        task.exceptionReady.connect(self._handleException)

        self._executor.submit(task)

    @Slot(object)
    def setBioMartConfiguration(self, configuration):
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        # parse the xml in the main thread (a long time ago this step was
        # done in a thread but would frequently cause `expat` to segfault.
        doc = biomart.parseXML(io.BytesIO(configuration))
        config = list(doc.elements("DatasetConfig"))[0]
        configuration = biomart.DatasetConfig(
            self.registry, config.tag, config.attributes, config.children)

        self.clearConfiguration()

        self.configuration = configuration

        def hidden(tree):
            return getattr(tree, "hidden", "false") != "false" or \
                   getattr(tree, "hideDisplay", "false") != "false"

        self.attributePagesTabWidget = tabs = gui.tabWidget(self.attributesConfigurationBox)

        for page in configuration.elements("AttributePage"):
            if not hidden(page):
                page_widget = PageWidget(page, self.dataset, self)
                gui.createTabPage(tabs, getattr(page, "displayName", ""),
                                  widgetToAdd=page_widget, canScroll=True)

        if self.SHOW_FILTERS:
            self.filterPagesTabWidget = tabs = gui.tabWidget(self.filtersConfigurationBox)
            for page in configuration.elements("FilterPage"):
                if not hidden(page):
                    page_widget = PageWidget(page, self.dataset, self)
                    gui.createTabPage(tabs, getattr(page, "displayName", ""),
                                      widgetToAdd=page_widget, canScroll=True)

        self.afterInit()

        self.commitButton.setEnabled(True)

    def clearConfiguration(self):
        self.mainTab.deleteLater()

        self.mainTab = QTabWidget()
        self.mainWidget.layout().addWidget(self.mainTab)
        self.mainWidget.layout().setCurrentWidget(self.mainTab)

        self.attributesConfigurationBox = gui.createTabPage(self.mainTab, "Attributes")
        if self.SHOW_FILTERS:
            self.filtersConfigurationBox = gui.createTabPage(self.mainTab, "Filters")

    def commit(self):
        pageconf = self.attributePagesTabWidget.currentWidget().widget()
        format = pageconf.outFormats

        self.error(100)
        if not self.hasBiopython and format.lower() == "fasta":
            self.error(100, "Cannot parse FASTA format")
            return

        query = pageconf.query()
        bydatasets = defaultdict(lambda: ([], []))

        for conftype, tree, val in query:
            dataset = self.dataset

            if conftype == "Attribute":
                bydatasets[dataset][0].append(tree.internalName)
            elif conftype == "Filter":
                bydatasets[dataset][1].append((tree.internalName, val))

        if self.SHOW_FILTERS:
            pageconf = self.filterPagesTabWidget.currentWidget().widget()
            query = pageconf.query()

            for conftype, tree, val in query:
                dataset = self.dataset

                if conftype == "Attribute":
                    bydatasets[dataset][0].append(tree.internalName)
                elif conftype == "Filter":
                    bydatasets[dataset][1].append((tree.internalName, val))

        query = self.registry.query(
            format="TSV" if "tsv" in format.lower() else format.upper(),
            uniqueRows=self.uniqueRows,
            virtualSchema=dataset.virtualSchema,
            serverVirtualSchema=dataset.serverVirtualSchema
        )

        for dataset, (attributes, filters) in bydatasets.items():
            query.set_dataset(dataset if dataset else self.dataset)
            for attr in attributes:
                query.add_attribute(attr)
            for filter, value in filters:
                query.add_filter(filter, value)

        self.error(0)
        self.setEnabled(False)
        self._task = task = concurrent.Task(function=query.get_table)
        task.resultReady.connect(self.dataReady)
        task.exceptionReady.connect(self._handleException)
        self._executor.submit(task)

    def dataReady(self, data):
        self.setEnabled(True)
        self.send("Data", data)

    def pushAction(self, action):
        ref = action.ref
        ref_widget = self.findChild(QWidget, ref)
        if hasattr(ref_widget, "setOptions"):
            ref_widget.setOptions(action.subelements_top("Option"))

    def registerDelayedCall(self, call):
        self._afterInitQueue.append(call)

    def afterInit(self):
        while self._afterInitQueue:
            call = self._afterInitQueue.pop(0)
            call()

    def clearCache(self):
        self.registry.connection.clear_cache()


def test_main(argv=sys.argv):
    from AnyQt.QtWidgets import QApplication
    app = QApplication(list(argv))

    ow = OWBioMart()
    ow.show()
    ow.raise_()

    r = app.exec_()
    ow.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())



"""dictyExpress widget"""
import sys
import os
import requests_cache


from requests.exceptions import HTTPError, ConnectionError
from PyQt4.QtGui import QTreeWidget, QTreeWidgetItem, QLineEdit, QLabel, QFont, QFrame, QApplication
from PyQt4.QtCore import Qt
from Orange.data import Table
from Orange.widgets.widget import OWWidget
from Orange.widgets import gui, settings
from orangecontrib.bio import resolwe
from .. import dicty
from ..utils import environ

CallBack = dicty.CallBack
Labels = [
    (" ", " "),
    ("var.project", "Project"),
    ("static.name", "Experiment"),
    ("static.cite", "Citation"),
    ("var.growth", "Growth"),
    ("var.treatment", "Treatment"),
    ("var.strain", "Parental strain")]


# Creates line separator
def h_line():
    line = QFrame()
    line.setFrameShape(QFrame.HLine)
    line.setFrameShadow(QFrame.Sunken)
    return line

#  Support cache with requests_cache module
cache_path = os.path.join(environ.buffer_dir, "resolwe")

try:
    os.makedirs(cache_path)
except OSError:
    pass

cache_file = os.path.join(cache_path, 'dictyExpress_cache')
requests_cache.install_cache(cache_name=cache_file, backend='sqlite')


class OWdictyExpress(OWWidget):

    name = "dictyExpress"
    description = "Time-course gene expression data"
    icon = "../widgets/icons/logo_dE.png"
    want_main_area = True
    priority = 37

    inputs = []
    outputs = [("Data", Table)]

    username = settings.Setting('anonymous@genialis.com')
    password = settings.Setting('')
    setTimeVariable = settings.Setting(False)

    def __init__(self):
        super().__init__()

        self.res = None
        self.server = 'https://dictyexpress.research.bcm.edu'
        self.headerLabels = [x[1] for x in Labels]
        self.searchString = ""
        self.items = []
        self.lastSelected = None  # store last selected customTreeItem

        # Login Section

        box = gui.widgetBox(self.controlArea, 'Login')

        self.namefield = gui.lineEdit(box, self, "username", "Username:", labelWidth=100, orientation='horizontal',
                                      callback=self.auth_changed)

        self.passfield = gui.lineEdit(box, self, "password", "Password:", labelWidth=100, orientation='horizontal',
                                      callback=self.auth_changed)

        self.passfield.setEchoMode(QLineEdit.Password)

        self.controlArea.layout().addWidget(h_line())

        self.time_var_checkBox = gui.checkBox(self.controlArea, self, "setTimeVariable",
                                              "Set Time variable")
        self.time_var_checkBox.setToolTip('Create new column where each row represents one time point')

        self.controlArea.layout().addWidget(h_line())

        self.commit_button = gui.button(self.controlArea, self, "Commit", callback=self.commit)
        self.handle_commit_button(False)

        self.refresh_button = gui.button(self.controlArea, self, "Refresh", callback=self.refresh)
        self.handle_cache_button(True)

        gui.rubber(self.controlArea)

        # Experiment Section

        label = QLabel("Available projects:")
        my_font = QFont()
        my_font.setBold(True)
        label.setFont(my_font)
        self.mainArea.layout().addWidget(label)

        self.mainArea.layout().addWidget(h_line())

        self.filter = gui.lineEdit(self.mainArea, self, "searchString", "Filter:", callbackOnType=True,
                                   callback=self.search_update)

        self.experimentsWidget = QTreeWidget(alternatingRowColors=True,
                                             rootIsDecorated=False,
                                             uniformRowHeights=True,
                                             sortingEnabled=True)

        self.experimentsWidget.setItemDelegateForColumn(
            0, gui.IndicatorItemDelegate(self, role=Qt.DisplayRole))

        self.experimentsWidget.selectionModel().selectionChanged.connect(
            self.onSelectionChanged)

        self.experimentsWidget.setHeaderLabels(self.headerLabels)
        self.mainArea.layout().addWidget(self.experimentsWidget)

        if self.username and self.password:
            self.connect()

    def auth_set(self):
        self.passfield.setDisabled(not self.username)

    def auth_changed(self):
        self.auth_set()
        self.connect()

    def refresh(self):
        self.reset()
        self.load_experiments()

    def reset(self):
        self.experimentsWidget.clear()  # clear QTreeWidget
        self.items = []
        self.lastSelected = None
        self.searchString = ""
        self.handle_commit_button(False)

    def search_update(self):
        parts = self.searchString.split()
        for item in self.items:
            item.setHidden(not all(s in item for s in parts))

    def connect(self):
        self.res = None
        self.error(1)
        self.reset()
        self.handle_commit_button(False)
        self.handle_cache_button(False)

        if self.username and self.password:
            try:
                self.res = resolwe.connect(self.username, self.password, self.server, 'genesis')
            except resolwe.ResolweAuthException as e:
                self.error(1, e.args[0])
            else:
                self.load_experiments()
                self.handle_cache_button(True)

    def load_experiments(self):
        with self.progressBar(100) as progress:
            if self.res:
                try:
                    experiments = self.res.fetch_etc_objects(progress.advance)  # returns a list of experiments
                except ConnectionError as e:
                    self.error(1, e.args[0])
                else:
                    self.load_tree_items(experiments)

    def load_tree_items(self, list_of_exp):
        self.items = [CustomTreeItem(self.experimentsWidget, item) for item in list_of_exp]
        for i in range(len(self.headerLabels)):
            self.experimentsWidget.resizeColumnToContents(i)

    def onSelectionChanged(self):
        self.handle_commit_button(True)

    def handle_commit_button(self, handle):
        self.commit_button.setEnabled(handle)

    def handle_cache_button(self, handle):
        self.refresh_button.setEnabled(handle)

    def commit(self):
        self.error(1)
        with self.progressBar(100) as progress:
            selected_item = self.experimentsWidget.currentItem()  # get selected TreeItem
            if self.lastSelected:
                self.lastSelected.setData(0, Qt.DisplayRole, "")
            self.lastSelected = selected_item
            selected_item.setData(0, Qt.DisplayRole, " ")

            try:
                etc_json = self.res.download_etc_data(selected_item.gen_data_id, progress.advance)
            except (HTTPError, ConnectionError, ValueError) as e:
                self.error(1, e.args[0])
            else:
                data = self.res.etc_to_table(etc_json, self.setTimeVariable,  progress.advance)
                self.send("Data", data)


class CustomTreeItem(QTreeWidgetItem):

    def __init__(self, parent, gen_data):
        super(CustomTreeItem, self).__init__(parent)  # Init super class (QtGui.QTreeWidgetItem )

        self._gen_data = gen_data  # GenData object
        self.set_rows(self._gen_data.annotation)  # set rows in QTreeWidget

    def __contains__(self, text):
        return any(text.upper() in str(self.text(i)).upper() for i in range(self.columnCount()))

    @property
    def gen_data_id(self):
        return self._gen_data.id

    def set_rows(self, row):
        for index, label in enumerate(Labels):
            if index > 0:
                if type(row[label[0]]["value"]) == list:
                    self.setText(index, row[label[0]]["value"][0]["name"])
                else:
                    self.setText(index, row[label[0]]["value"])


def test_main():
    app = QApplication(sys.argv)
    dicty.verbose = True
    w = OWdictyExpress()
    w.show()
    r = app.exec_()
    w.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())



import doctest

from orangecontrib.bio import arrayexpress


def load_tests(loader, tests, ignore):
    conn = arrayexpress.ArrayExpressConnection()
    foo = type("foo", (object,), {})()
    bar = type("bar", (object,), {})()

    tests.addTests(
        doctest.DocTestSuite(
            arrayexpress,
            extraglobs={"conn": conn, "foo": foo, "bar": bar},
            optionflags=doctest.ELLIPSIS)
    )
    return tests



from __future__ import absolute_import, division

import os
import sys
import warnings

try:
    import cPickle as pickle
except ImportError:
    import pickle

try:
    from Orange.utils import environ
except ImportError:
    from orangecontrib.bio.utils import environ

from orangecontrib.bio.utils import serverfiles

_COMMON_NAMES = (
    ("3702",   "Arabidopsis thaliana"),
    ("9913",   "Bos taurus"),
    ("6239",   "Caenorhabditis elegans"),
    ("3055",   "Chlamydomonas reinhardtii"),
    ("7955",   "Danio rerio"),
    ("352472", "Dictyostelium discoideum AX4"),
    ("7227",   "Drosophila melanogaster"),
    ("562",    "Escherichia coli"),
    ("11103",  "Hepatitis C virus"),
    ("9606",   "Homo sapiens"),
    ("10090",  "Mus musculus"),
    ("2104",   "Mycoplasma pneumoniae"),
    ("4530",   "Oryza sativa"),
    ("5833",   "Plasmodium falciparum"),
    ("4754",   "Pneumocystis carinii"),
    ("10116",  "Rattus norvegicus"),
    ("4932",   "Saccharomyces cerevisiae"),
    ("4896",   "Schizosaccharomyces pombe"),
    ("31033",  "Takifugu rubripes"),
    ("8355",   "Xenopus laevis"),
    ("4577",   "Zea mays"),
    ("5476",   "Candida albicans")
)

_COMMON_NAMES_MAPPING = dict(_COMMON_NAMES)


# list of common organisms from http://www.ncbi.nlm.nih.gov/Taxonomy
def common_taxids():
    """Return taxonomy IDs for common organisms."""
    # Sorted lexicographically by names
    return [taxid for taxid, _ in _COMMON_NAMES]


def common_taxid_to_name(taxid):
    """Return a name for a common organism taxonomy id."""
    return _COMMON_NAMES_MAPPING[taxid]


def taxname_to_taxid(name):
    """Return taxonomy ID for a taxonomy name."""
    name_to_taxid = dict(map(reversed, _COMMON_NAMES))

    if name in name_to_taxid:
        return name_to_taxid[name]
    return None


def essential_taxids():
    """Return taxonomy IDs for organisms that are included in (default)
    Orange Bioinformatics installation.
    """
    return ["352472", # Dictyostelium discoideum
            "7227",  # Drosophila melanogaster
            "9606",  # Homo sapiens
            "10090", # Mus musculus
            "4932",  # Saccharomyces cerevisiae
            ]


def shortname(taxid):
    """ Short names for common_taxids organisms """
    names = {
    "3702"  : ["arabidopsis", "thaliana", "plant"],
    "9913"  : ["cattle", "cow"],
    "6239"  : ["nematode", "roundworm"],
    "3055"  : ["algae"],
    "7955"  : ["zebrafish"],
    "352472": ["dicty", "amoeba", "slime mold"],
    "7227"  : ["fly", "fruit fly", "vinegar fly"],
    "562"   : ["ecoli", "coli", "bacterium"],
    "11103" : ["virus, hepatitis"],
    "9606"  : ["human"],
    "10090" : ["mouse", "mus"],
    "2104"  : ["bacterium", "mycoplasma"],
    "4530"  : ["asian rice", "rice", "cereal", "plant"],
    "5833"  : ["plasmodium", "malaria", "parasite"],
    "4754"  : ["pneumonia", "fungus"],
    "10116" : ["rat", "laboratory rat"],
    "4932"  : ["yeast", "baker yeast", "brewer yeast"],
    "4896"  : ["yeast", "fission yeast"],
    "31033" : ["fish", "pufferfish"],
    "8355"  : ["frog", "african clawed frog"],
    "4577"  : ["corn", "cereal grain", "plant"]
    }
    if taxid in names:
        return names[taxid]
    return []


class MultipleSpeciesException(Exception):
    pass


class UnknownSpeciesIdentifier(Exception):
    pass


def pickled_cache(filename=None, dependencies=[], version=1, maxSize=30,
                  pickleprotocol=pickle.HIGHEST_PROTOCOL):
    """
    Return a persistent cache function decorator.
    """
    def datetime_info(domain, filename):
        try:
            return serverfiles.info(domain, filename)["datetime"]
        except IOError:
            return serverfiles.ServerFiles().info(domain, filename)["datetime"]

    def cached(func):
        pytag = "py{0}.{1}".format(*sys.version_info[:2])
        if filename is None:
            cache_filename = os.path.join(
                environ.buffer_dir, func.__module__ + "_" + func.__name__ +
                "_" + pytag + "_cache.pickle")
        else:
            cache_filename = filename

        def f(*args, **kwargs):
            currentVersion = tuple([datetime_info(domain, file)
                                    for domain, file in dependencies] +
                                    [version, pytag])
            try:
                with open(cache_filename, "rb") as f:
                    cachedVersion, cache = pickle.load(f)
            except (OSError, IOError) as er:
                cachedVersion, cache = "no version", {}
            except ValueError as er:
                # unknown pickle format
                cachedVersion, cache = "no version", {}
            except Exception as er:
                warnings.warn(
                    "An error occurred while reading cache, using empty cache",
                    UserWarning)
                cachedVersion, cache = "no version", {}
            else:
                if cachedVersion != currentVersion:
                    cachedVersion, cache = "no version", {}

            allArgs = args + tuple([(key, tuple(value) if type(value) in [set, list] else value)\
                                     for key, value in kwargs.items()])
            if allArgs in cache:
                return cache[allArgs]
            else:
                res = func(*args, **kwargs)
                if len(cache) > maxSize:
                    del cache[next(iter(cache))]
                cache[allArgs] = res
                try:
                    with open(cache_filename, "wb") as f:
                        pickle.dump((currentVersion, cache), f,
                                    protocol=pickleprotocol)
                except (OSError, IOError) as err:
                    pass

                return res
        return f

    return cached


class Taxonomy(object):
    DOMAIN = "Taxonomy"
    FILENAME = "ncbi-taxonomy.sqlite"

    def __init__(self):
        from .ncbi.taxonomy import Taxonomy
        # Ensure the taxonomy db is downloaded.
        filename = serverfiles.localpath_download(self.DOMAIN, self.FILENAME)
        self._tax = Taxonomy(filename)

    def get_entry(self, id):
        try:
            return self._tax[id]
        except KeyError:
            raise UnknownSpeciesIdentifier(id)

    def search(self, string, onlySpecies=True, exact=False):
        res = self._tax.search(string, exact)
        if onlySpecies:
            res = [taxid for taxid in res
                   if self._tax[taxid].rank == "species"]
        return res

    def __iter__(self):
        return iter(self._tax)

    def __getitem__(self, id):
        return self.get_entry(id).name

    def other_names(self, id):
        return [(name, q) for name, q in self._tax[id].synonyms
                if q != "scientific name"]

    def rank(self, id):
        return self._tax[id].rank

    def parent(self, id):
        return self._tax[id].parent_tax_id

    def subnodes(self, id, levels=1):
        res = self._tax.child_tax_ids(id)
        if levels > 1:
            for child_id in res:
                res.extend(self.subnodes(child_id, levels - 1))
        return res

    def taxids(self):
        return list(self._tax)


@pickled_cache(None, [("Taxonomy", "ncbi-taxonomy.sqlite")], version=2)
def name(taxid):
    """
    Return the scientific name for organism with taxid.
    """
    # Most of the lookups will be for the common names, so in most
    # situations we can avoid loading the taxonomy.
    if taxid in _COMMON_NAMES:
        return _COMMON_NAMES[taxid]
    else:
        return Taxonomy()[taxid]


@pickled_cache(None, [("Taxonomy", "ncbi-taxonomy.sqlite")], version=2)
def other_names(taxid):
    """
    Return a list of (name, name_type) tuples excluding the scientific name.

    Use :func:`name` to retrieve the scientific name.

    """
    return  Taxonomy().other_names(taxid)


@pickled_cache(None, [("Taxonomy", "ncbi-taxonomy.sqlite")], version=2)
def search(string, onlySpecies=True, exact=False):
    """ Search the NCBI taxonomy database for an organism.

    :param string: Search string.
    :param onlySpecies: Return only taxids of species (and subspecies).
    :param exact:  Return only taxids of organism that exactly match the string.
    """
    ids = Taxonomy().search(string, onlySpecies, exact)
    return list(ids)


@pickled_cache(None, [("Taxonomy", "ncbi-taxonomy.sqlite")], version=2)
def lineage(taxid):
    """ Return a list of taxids ordered from the topmost node (root) to taxid.
    """
    return Taxonomy()._tax.lineage(taxid)


def to_taxid(code, mapTo=None):
    """
    See if the code is a valid code in GO or KEGG and return a set of its taxids.
    """
    warnings.warn("'to_taxid' is deprecated", DeprecationWarning)

    try:
        name(code)
        results = set([code])
    except UnknownSpeciesIdentifier:
        results = set()
        from . import kegg, go
        for test in [kegg.to_taxid, go.to_taxid]:
            try:
                r = test(code)
                if type(r) == set:
                    results.update(r)
                elif r:
                    results.add(r)
            except Exception:
                pass

    if mapTo:
        mapTo = set(mapTo)
        if mapTo.intersection(results):
            return mapTo.intersection(results)

        mapped = set()
        for r in results:
            r_lin = lineage(r)
            if mapTo.intersection(r_lin):
                mapped.update(mapTo.intersection(r_lin))

        if not mapped:
            for m in mapTo:
                m_lin = lineage(m)
                if results.intersection(m_lin):
                    mapped.add(m)
        results = mapped

    return results


def taxids():
    """
    Returns a list of all (about half a million!) NCBI's taxonomy ID's.
    """
    return Taxonomy().taxids()


def ensure_downloaded(callback=None, verbose=True):
    """
    Retrieve the taxonomy database if not already downloaded.
    """
    serverfiles.localpath_download(
        Taxonomy.DOMAIN, Taxonomy.FILENAME,
        callback=callback, verbose=verbose)


if __name__ == "__main__":
    ids = search("Homo sapiens")
    print(ids)
    print(other_names(ids[0]))



import os
import sqlite3
import tarfile
import shutil
import tempfile
import collections
import textwrap

from collections import namedtuple

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

import six

__all__ = ["Taxonomy"]

pjoin = os.path.join

TAXDUMP_URL = "ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz"

_taxon = namedtuple(
    "taxon",
    ["tax_id", "parent_tax_id", "name", "synonyms", "rank"]
)


def namedtuple_repr_pretty(self, p, cycle):
    name = type(self).__name__
    if cycle:
        p.text("{0}(...)".format("name"))
    else:
        with p.group(len(name) + 1, "{0}(".format(name), ")"):
            for field, val in zip(self._fields, self):
                p.text(field + "=")
                p.pretty(val)
                if field != self._fields[-1]:
                    p.text(",")
                    p.breakable()


class taxon(_taxon):
    _repr_pretty_ = namedtuple_repr_pretty


class Taxonomy(collections.Mapping):
    SCHEMA_VERSION = (0, 0, 1)

    def __init__(self, taxdb):
        self._con = sqlite3.connect(taxdb, timeout=15)
        self._con.execute("""
            CREATE INDEX IF NOT EXISTS
                index_names_tax_id ON names(tax_id)
        """)

    def __node_query(self, tax_id):
        c = self._con.execute("""
            SELECT nodes.tax_id, nodes.parent_tax_id, ranks.rank
            FROM nodes INNER JOIN ranks USING(rank_id)
            WHERE tax_id = ?
        """, (tax_id,))
        if not c:
            raise KeyError(tax_id)
        else:
            return next(c)

    def __getitem__(self, tax_id):
        if not isinstance(tax_id, six.string_types):
            raise TypeError("expected a string, got {}"
                            .format(type(tax_id).__name__))

        c = self._con.execute("""
            SELECT names.name, name_classes.name_class
            FROM names INNER JOIN name_classes USING (name_class_id)
            WHERE names.tax_id = ?
        """, (tax_id,))

        names = list(c)
        if not names:
            raise KeyError(tax_id)

        scientific_name = None
        for name, name_class in names:
            if name_class == "scientific name":
                scientific_name = name
        node = self.__node_query(tax_id)
        _, parent, rank = node
        return taxon(str(tax_id), str(parent), scientific_name, names, rank)

    def __iter__(self):
        c = self._con.execute("SELECT tax_id FROM nodes")
        return (str(r[0]) for r in c)

    def __len__(self):
        c = self._con.execute("SELECT COUNT(*) FROM nodes")
        return next(c)[0]

    def search(self, name, exact=True):
        # First ensure the name column is indexed.
        self._con.execute("""
            CREATE INDEX IF NOT EXISTS
                index_names_name ON names(name)""")

        if not exact:
            name = "{0}%".format(name)
            operator = "LIKE"
        else:
            operator = "="

        c = self._con.execute("""
            SELECT DISTINCT(tax_id)
            FROM names
            WHERE names.name {operator} ?
            """.format(operator=operator), (name,))
        return (str(r[0]) for r in c)

    def lineage(self, tax_id):
        lineage = []
        while True:
            parent = self.parent_tax_id(tax_id)
            if parent is not None:
                lineage.append(parent)
                tax_id = parent
            else:
                break
        return list(reversed(lineage))

    def parent_tax_id(self, tax_id):
        if not isinstance(tax_id, six.string_types):
            raise TypeError("Expected a string")

        node = self.__node_query(tax_id)
        _, parent, _ = node
        if parent == int(tax_id):
            # The root (with tax_id 1) has itself as a parent
            return None
        else:
            return str(parent)

    def child_tax_ids(self, tax_id):
        if not isinstance(tax_id, six.string_types):
            raise TypeError("Expected a string")

        c = self._con.execute("""
            SELECT tax_id
            FROM nodes
            WHERE parent_tax_id = ?
        """, (tax_id,))
        children = list(str(r[0]) for r in c if r[0] != 1)
        return children

    def name(self, tax_id):
        return self[tax_id].name

    def synonyms(self, tax_id):
        return self[tax_id].synonyms

    @classmethod
    def initialize(cls, db_filename, taxdump=None):

        tempd = None
        if taxdump is None:
            tempd = tempfile.mkdtemp()
            cls.download(tempd)
            taxdump = pjoin(tempd, "taxdump.tar.gz")

        try:
            cls.init_db(db_filename, tarfile.open(taxdump))
        finally:
            if tempd is not None:
                shutil.rmtree(tempd)

    @classmethod
    def download(cls, download_dir):
        """
        Download the taxonomy archive from the ncbi ftp server.

        :param str download_dir: Download directory.

        """
        stream = urlopen(TAXDUMP_URL, timeout=30)

        if not os.path.isdir(download_dir):
            os.makedirs(download_dir)

        with open(pjoin(download_dir, "taxdump.tar.gz"), "wb") as f:
            shutil.copyfileobj(stream, f)

    @classmethod
    def init_db(cls, dbfilename, taxdump):

        con = sqlite3.connect(dbfilename)
        cursor = con.cursor()

        indexes = ["index_names_tax_id",
                   "index_names_name"]

        for index in indexes:
            cursor.execute("DROP INDEX IF EXISTS %s" % index)

        for table in ["nodes", "name_classes", "names", "ranks"]:
            cursor.execute("DROP TABLE IF EXISTS %s" % table)

        script = textwrap.dedent("""
            CREATE TABLE ranks (
                rank_id INTEGER PRIMARY KEY ASC,
                rank TEXT UNIQUE
            );

            CREATE TABLE name_classes (
                name_class_id INTEGER PRIMARY KEY ASC,
                name_class TEXT UNIQUE
            );

            CREATE TABLE nodes (
                tax_id INTEGER PRIMARY KEY ASC,
                parent_tax_id INTEGER,
                rank_id INTEGER REFERENCES ranks(rank_id)
            );

            CREATE TABLE names (
                tax_id INTEGER REFERENCES nodes(tax_id),
                name text COLLATE NOCASE,
                name_class_id INTEGER REFERENCES name_classes(name_class_id)
            );
        """)

        cursor.executescript(script)

        nodes = taxdump.extractfile("nodes.dmp")

        def iter_rows(lines):
            for line in lines:
                if not line.strip():
                    continue
                yield tuple(line.decode("utf-8").rstrip("\t\n|").split("\t|\t"))

        # take the tax_id, parent_tax_id, rank entries
        nodes = [row[:3] for row in iter_rows(nodes)]

        ranks = list(enumerate(sorted(set(rank for _, _, rank in nodes))))
        rank_id = {rank: i for i, rank in ranks}

        cursor.executemany("INSERT INTO ranks VALUES (?, ?)", ranks)

        cursor.executemany("INSERT INTO nodes VALUES (?, ?, ?)",
                           ((int(tax_id), int(parent_tax_id), rank_id[rank])
                            for tax_id, parent_tax_id, rank in nodes))

        names = taxdump.extractfile("names.dmp")

        def iter_names(rows):
            for tax_id, name, unique_name, name_class in rows:
                if unique_name:
                    name = unique_name
                yield tax_id, name, name_class

        names = list(iter_names(iter_rows(names)))
        name_classes = list(enumerate(set(name_class
                                          for _, _, name_class in names)))
        name_class_id = {name_class: i for i, name_class in name_classes}

        cursor.executemany("INSERT INTO name_classes VALUES (?, ?)",
                           name_classes)
        cursor.executemany("INSERT INTO names VALUES (?, ?, ?)",
                           ((int(tax_id), name, name_class_id[name_class])
                            for tax_id, name, name_class in names))

        con.commit()
        con.close()



from __future__ import absolute_import

from . import stats
from . import expression
from . import group

def progress_bar_milestones(count, iterations=100):
    return set([int(i*count/float(iterations)) for i in range(iterations)])



import doctest
import unittest

from six import StringIO

from orangecontrib.bio import ontology

class TestOntology(unittest.TestCase):
    def test_oboobject(self):
        stanza = '''[Term]
id: FOO:001
name: bar
'''
        term = ontology.OBOObject.parse_stanza(stanza)

#        self.assertEqual(term.get_value("id"), "FOO:001")
#        self.assertEqual(term.get_value("name"), "bar")

        term.add_tag("tt", "3")

    def test_ontology(self):
        seinfeld = StringIO("""
[Typedef]
id: parent

[Typedef]
id: child
inverse_of: parent ! not actually used yet

[Term]
id: 001
name: George

[Term]
id: 002
name: Estelle
relationship: parent 001 ! George

[Term]
id: 003
name: Frank
relationship: parent 001 ! George

""")  # TODO: fill the ontology with all characters
        stanza = '''[Term]
id: FOO:001
name: bar
'''
#        term = ontology.OBOObject.parse_stanza(stanza)

        seinfeld = ontology.OBOOntology(seinfeld)
#        print(seinfeld.child_edges("001"))

def load_tests(loader, tests, ignore):
    stanza = '''[Term]
id: FOO:001
name: bar
'''
    term = ontology.OBOObject.parse_stanza(stanza)
    tests.addTests(doctest.DocTestSuite(ontology,
                                        extraglobs={"term": term},
                                        optionflags=doctest.ELLIPSIS))
    return tests



import contextlib
import itertools
from itertools import chain

from typing import Iterable, Generator, Tuple

from AnyQt.QtCore import Qt

__all__ = ["group_ranges", "disconnected"]


@contextlib.contextmanager
def disconnected(boundsig, slot, conntype=Qt.UniqueConnection):
    """
    Disconnect a slot from a bound signal

    Parameters
    ----------
    boundsig : pyqtBoundSignal
    slot : pyqtBoundSlot
    conntype : Qt.ConnectionType

    """
    boundsig.disconnect(slot)
    try:
        yield
    finally:
        boundsig.connect(slot, conntype)


def group_ranges(indices):
    # type: (Iterable[int]) -> Iterable[Tuple[int, int]]
    """
    Group consecutive indices into `(start, stop)` tuple ranges.

    Parameters
    ----------
    indices : Iterable[int]
        An iterable yielding indices

    Example
    -------
    >>> list(group_ranges([1, 2, 3, 5, 3, 4]))
    [(1, 4), (5, 6), (3, 5)]
    >>> seq = [1, 2, 3, 3, 3, 5, 6, 9, 8]
    >>> assert seq == list(chain.from_iterable(
    ...        (range(s, e) for s, e in group_ranges(seq))))
    ...
    """
    g = itertools.groupby(enumerate(indices),
                          key=lambda t: t[1] - t[0])
    for _, range_ind in g:
        range_ind = list(range_ind)
        _, start = range_ind[0]
        _, end = range_ind[-1]
        yield start, end + 1
